resources_types = {
    "aws": [
        "aws",
        "aws_root",
        "aws_root_access_key",
        "aws_acm_certificate",
        "aws_acm_certificate_validation",
        "aws_acmpca_certificate_authority",
        "aws_api_gateway_account",
        "aws_api_gateway_api_key",
        "aws_api_gateway_authorizer",
        "aws_api_gateway_base_path_mapping",
        "aws_api_gateway_client_certificate",
        "aws_api_gateway_deployment",
        "aws_api_gateway_documentation_part",
        "aws_api_gateway_documentation_version",
        "aws_api_gateway_domain_name",
        "aws_api_gateway_gateway_response",
        "aws_api_gateway_integration",
        "aws_api_gateway_integration_response",
        "aws_api_gateway_method",
        "aws_api_gateway_method_response",
        "aws_api_gateway_method_settings",
        "aws_api_gateway_model",
        "aws_api_gateway_request_validator",
        "aws_api_gateway_resource",
        "aws_api_gateway_rest_api",
        "aws_api_gateway_stage",
        "aws_api_gateway_usage_plan",
        "aws_api_gateway_usage_plan_key",
        "aws_api_gateway_vpc_link",
        "aws_apigatewayv2_api",
        "aws_apigatewayv2_api_mapping",
        "aws_apigatewayv2_authorizer",
        "aws_apigatewayv2_deployment",
        "aws_apigatewayv2_domain_name",
        "aws_apigatewayv2_integration",
        "aws_apigatewayv2_integration_response",
        "aws_apigatewayv2_model",
        "aws_apigatewayv2_route",
        "aws_apigatewayv2_route_response",
        "aws_apigatewayv2_stage",
        "aws_apigatewayv2_vpc_link",
        "aws_accessanalyzer_analyzer",
        "aws_appmesh_mesh",
        "aws_appmesh_route",
        "aws_appmesh_virtual_node",
        "aws_appmesh_virtual_router",
        "aws_appmesh_virtual_service",
        "aws_appsync_api_key",
        "aws_appsync_datasource",
        "aws_appsync_function",
        "aws_appsync_graphql_api",
        "aws_appsync_resolver",
        "aws_appautoscaling_policy",
        "aws_appautoscaling_scheduled_action",
        "aws_appautoscaling_target",
        "aws_athena_database",
        "aws_athena_named_query",
        "aws_athena_workgroup",
        "aws_autoscaling_attachment",
        "aws_autoscaling_group",
        "aws_autoscaling_lifecycle_hook",
        "aws_autoscaling_notification",
        "aws_autoscaling_policy",
        "aws_autoscaling_schedule",
        "aws_backup_plan",
        "aws_backup_selection",
        "aws_backup_vault",
        "aws_batch_compute_environment",
        "aws_batch_job_definition",
        "aws_batch_job_queue",
        "aws_budgets_budget",
        "aws_cloud9_environment_ec2",
        "aws_cloudformation_stack",
        "aws_cloudformation_stack_set",
        "aws_cloudformation_stack_set_instance",
        "aws_cloudfront_distribution",
        "aws_cloudfront_origin_access_identity",
        "aws_cloudfront_public_key",
        "aws_cloudhsm_v2_cluster",
        "aws_cloudhsm_v2_hsm",
        "aws_cloudtrail",
        "aws_cloudwatch_dashboard",
        "aws_cloudwatch_event_permission",
        "aws_cloudwatch_event_rule",
        "aws_cloudwatch_event_target",
        "aws_cloudwatch_log_destination",
        "aws_cloudwatch_log_destination_policy",
        "aws_cloudwatch_log_group",
        "aws_cloudwatch_log_metric_filter",
        "aws_cloudwatch_log_resource_policy",
        "aws_cloudwatch_log_stream",
        "aws_cloudwatch_log_subscription_filter",
        "aws_cloudwatch_metric_alarm",
        "aws_codebuild_project",
        "aws_codebuild_source_credential",
        "aws_codebuild_webhook",
        "aws_codecommit_repository",
        "aws_codecommit_trigger",
        "aws_codedeploy_app",
        "aws_codedeploy_deployment_config",
        "aws_codedeploy_deployment_group",
        "aws_codepipeline",
        "aws_codepipeline_webhook",
        "aws_codestarnotifications_notification_rule",
        "aws_cognito_identity_pool",
        "aws_cognito_identity_pool_roles_attachment",
        "aws_cognito_identity_provider",
        "aws_cognito_resource_server",
        "aws_cognito_user_group",
        "aws_cognito_user_pool",
        "aws_cognito_user_pool_client",
        "aws_cognito_user_pool_domain",
        "aws_config_aggregate_authorization",
        "aws_config_config_rule",
        "aws_config_configuration_aggregator",
        "aws_config_configuration_recorder",
        "aws_config_configuration_recorder_status",
        "aws_config_delivery_channel",
        "aws_config_organization_custom_rule",
        "aws_config_organization_managed_rule",
        "aws_cur_report_definition",
        "aws_dlm_lifecycle_policy",
        "aws_datapipeline_pipeline",
        "aws_datasync_agent",
        "aws_datasync_location_efs",
        "aws_datasync_location_nfs",
        "aws_datasync_location_s3",
        "aws_datasync_location_smb",
        "aws_datasync_task",
        "aws_dms_certificate",
        "aws_dms_endpoint",
        "aws_dms_event_subscription",
        "aws_dms_replication_instance",
        "aws_dms_replication_subnet_group",
        "aws_dms_replication_task",
        "aws_devicefarm_project",
        "aws_dx_bgp_peer",
        "aws_dx_connection",
        "aws_dx_connection_association",
        "aws_dx_gateway",
        "aws_dx_gateway_association",
        "aws_dx_gateway_association_proposal",
        "aws_dx_hosted_private_virtual_interface",
        "aws_dx_hosted_private_virtual_interface_accepter",
        "aws_dx_hosted_public_virtual_interface",
        "aws_dx_hosted_public_virtual_interface_accepter",
        "aws_dx_hosted_transit_virtual_interface",
        "aws_dx_hosted_transit_virtual_interface_accepter",
        "aws_dx_lag",
        "aws_dx_private_virtual_interface",
        "aws_dx_public_virtual_interface",
        "aws_dx_transit_virtual_interface",
        "aws_directory_service_conditional_forwarder",
        "aws_directory_service_directory",
        "aws_directory_service_log_subscription",
        "aws_docdb_cluster",
        "aws_docdb_cluster_instance",
        "aws_docdb_cluster_parameter_group",
        "aws_docdb_cluster_snapshot",
        "aws_docdb_subnet_group",
        "aws_dynamodb_global_table",
        "aws_dynamodb_table",
        "aws_dynamodb_table_item",
        "aws_dax_cluster",
        "aws_dax_parameter_group",
        "aws_dax_subnet_group",
        "aws_ami",
        "aws_ami_copy",
        "aws_ami_from_instance",
        "aws_ami_launch_permission",
        "aws_ebs_default_kms_key",
        "aws_ebs_encryption_by_default",
        "aws_ebs_snapshot",
        "aws_ebs_snapshot_copy",
        "aws_ebs_volume",
        "aws_ec2_availability_zone_group",
        "aws_ec2_capacity_reservation",
        "aws_ec2_client_vpn_authorization_rule",
        "aws_ec2_client_vpn_endpoint",
        "aws_ec2_client_vpn_network_association",
        "aws_ec2_client_vpn_route",
        "aws_ec2_fleet",
        "aws_ec2_local_gateway_route",
        "aws_ec2_local_gateway_route_table_vpc_association",
        "aws_ec2_tag",
        "aws_ec2_traffic_mirror_filter",
        "aws_ec2_traffic_mirror_filter_rule",
        "aws_ec2_traffic_mirror_session",
        "aws_ec2_traffic_mirror_target",
        "aws_ec2_transit_gateway",
        "aws_ec2_transit_gateway_peering_attachment",
        "aws_ec2_transit_gateway_peering_attachment_accepter",
        "aws_ec2_transit_gateway_route",
        "aws_ec2_transit_gateway_route_table",
        "aws_ec2_transit_gateway_route_table_association",
        "aws_ec2_transit_gateway_route_table_propagation",
        "aws_ec2_transit_gateway_vpc_attachment",
        "aws_ec2_transit_gateway_vpc_attachment_accepter",
        "aws_eip",
        "aws_eip_association",
        "aws_instance",
        "aws_key_pair",
        "aws_launch_configuration",
        "aws_launch_template",
        "aws_placement_group",
        "aws_snapshot_create_volume_permission",
        "aws_spot_datafeed_subscription",
        "aws_spot_fleet_request",
        "aws_spot_instance_request",
        "aws_volume_attachment",
        "aws_ecr_lifecycle_policy",
        "aws_ecr_repository",
        "aws_ecr_repository_policy",
        "aws_ecs_capacity_provider",
        "aws_ecs_cluster",
        "aws_ecs_service",
        "aws_ecs_task_definition",
        "aws_efs_access_point",
        "aws_efs_file_system",
        "aws_efs_file_system_policy",
        "aws_efs_mount_target",
        "aws_eks_cluster",
        "aws_eks_fargate_profile",
        "aws_eks_node_group",
        "aws_elasticache_cluster",
        "aws_elasticache_parameter_group",
        "aws_elasticache_replication_group",
        "aws_elasticache_security_group",
        "aws_elasticache_subnet_group",
        "aws_elastic_beanstalk_application",
        "aws_elastic_beanstalk_application_version",
        "aws_elastic_beanstalk_configuration_template",
        "aws_elastic_beanstalk_environment",
        "aws_app_cookie_stickiness_policy",
        "aws_elb",
        "aws_elb_attachment",
        "aws_lb_cookie_stickiness_policy",
        "aws_lb_ssl_negotiation_policy",
        "aws_load_balancer_backend_server_policy",
        "aws_load_balancer_listener_policy",
        "aws_load_balancer_policy",
        "aws_proxy_protocol_policy",
        "aws_lb",
        "aws_lb_listener",
        "aws_lb_listener_certificate",
        "aws_lb_listener_rule",
        "aws_lb_target_group",
        "aws_lb_target_group_attachment",
        "aws_emr_cluster",
        "aws_emr_instance_group",
        "aws_emr_security_configuration",
        "aws_elastictranscoder_pipeline",
        "aws_elastictranscoder_preset",
        "aws_elasticsearch_domain",
        "aws_elasticsearch_domain_policy",
        "aws_fsx_lustre_file_system",
        "aws_fsx_windows_file_system",
        "aws_fms_admin_account",
        "aws_gamelift_alias",
        "aws_gamelift_build",
        "aws_gamelift_fleet",
        "aws_gamelift_game_session_queue",
        "aws_glacier_vault",
        "aws_glacier_vault_lock",
        "aws_globalaccelerator_accelerator",
        "aws_globalaccelerator_endpoint_group",
        "aws_globalaccelerator_listener",
        "aws_glue_catalog_database",
        "aws_glue_catalog_table",
        "aws_glue_classifier",
        "aws_glue_connection",
        "aws_glue_crawler",
        "aws_glue_job",
        "aws_glue_security_configuration",
        "aws_glue_trigger",
        "aws_glue_workflow",
        "aws_guardduty_detector",
        "aws_guardduty_invite_accepter",
        "aws_guardduty_ipset",
        "aws_guardduty_member",
        "aws_guardduty_organization_admin_account",
        "aws_guardduty_organization_configuration",
        "aws_guardduty_threatintelset",
        "aws_iam_access_key",
        "aws_iam_account_alias",
        "aws_iam_account_password_policy",
        "aws_iam_group",
        "aws_iam_group_membership",
        "aws_iam_group_policy",
        "aws_iam_group_policy_attachment",
        "aws_iam_instance_profile",
        "aws_iam_openid_connect_provider",
        "aws_iam_policy",
        "aws_iam_policy_attachment",
        "aws_iam_policy_document",
        "aws_iam_role",
        "aws_iam_role_policy",
        "aws_iam_role_policy_attachment",
        "aws_iam_saml_provider",
        "aws_iam_server_certificate",
        "aws_iam_service_linked_role",
        "aws_iam_user",
        "aws_iam_user_group_membership",
        "aws_iam_user_login_profile",
        "aws_iam_user_policy",
        "aws_iam_user_policy_attachment",
        "aws_iam_user_ssh_key",
        "aws_inspector_assessment_target",
        "aws_inspector_assessment_template",
        "aws_inspector_resource_group",
        "aws_iot_certificate",
        "aws_iot_policy",
        "aws_iot_policy_attachment",
        "aws_iot_role_alias",
        "aws_iot_thing",
        "aws_iot_thing_principal_attachment",
        "aws_iot_thing_type",
        "aws_iot_topic_rule",
        "aws_kms_alias",
        "aws_kms_ciphertext",
        "aws_kms_external_key",
        "aws_kms_grant",
        "aws_kms_key",
        "aws_kinesis_analytics_application",
        "aws_kinesis_stream",
        "aws_kinesis_firehose_delivery_stream",
        "aws_kinesis_video_stream",
        "aws_lambda_alias",
        "aws_lambda_event_source_mapping",
        "aws_lambda_function",
        "aws_lambda_function_event_invoke_config",
        "aws_lambda_layer_version",
        "aws_lambda_permission",
        "aws_lambda_provisioned_concurrency_config",
        "aws_licensemanager_association",
        "aws_licensemanager_license_configuration",
        "aws_lightsail_domain",
        "aws_lightsail_instance",
        "aws_lightsail_key_pair",
        "aws_lightsail_static_ip",
        "aws_lightsail_static_ip_attachment",
        "aws_mq_broker",
        "aws_mq_configuration",
        "aws_macie_member_account_association",
        "aws_macie_s3_bucket_association",
        "aws_msk_cluster",
        "aws_msk_configuration",
        "aws_media_convert_queue",
        "aws_media_package_channel",
        "aws_media_store_container",
        "aws_media_store_container_policy",
        "aws_neptune_cluster",
        "aws_neptune_cluster_instance",
        "aws_neptune_cluster_parameter_group",
        "aws_neptune_cluster_snapshot",
        "aws_neptune_event_subscription",
        "aws_neptune_parameter_group",
        "aws_neptune_subnet_group",
        "aws_opsworks_application",
        "aws_opsworks_custom_layer",
        "aws_opsworks_ganglia_layer",
        "aws_opsworks_haproxy_layer",
        "aws_opsworks_instance",
        "aws_opsworks_java_app_layer",
        "aws_opsworks_memcached_layer",
        "aws_opsworks_mysql_layer",
        "aws_opsworks_nodejs_app_layer",
        "aws_opsworks_permission",
        "aws_opsworks_php_app_layer",
        "aws_opsworks_rails_app_layer",
        "aws_opsworks_rds_db_instance",
        "aws_opsworks_stack",
        "aws_opsworks_static_web_layer",
        "aws_opsworks_user_profile",
        "aws_organizations_account",
        "aws_organizations_organization",
        "aws_organizations_organizational_unit",
        "aws_organizations_policy",
        "aws_organizations_policy_attachment",
        "aws_pinpoint_adm_channel",
        "aws_pinpoint_apns_channel",
        "aws_pinpoint_apns_sandbox_channel",
        "aws_pinpoint_apns_voip_channel",
        "aws_pinpoint_apns_voip_sandbox_channel",
        "aws_pinpoint_app",
        "aws_pinpoint_baidu_channel",
        "aws_pinpoint_email_channel",
        "aws_pinpoint_event_stream",
        "aws_pinpoint_gcm_channel",
        "aws_pinpoint_sms_channel",
        "aws_qldb_ledger",
        "aws_quicksight_group",
        "aws_quicksight_user",
        "aws_ram_principal_association",
        "aws_ram_resource_association",
        "aws_ram_resource_share",
        "aws_ram_resource_share_accepter",
        "aws_db_cluster_snapshot",
        "aws_db_event_subscription",
        "aws_db_instance",
        "aws_db_instance_role_association",
        "aws_db_option_group",
        "aws_db_parameter_group",
        "aws_db_security_group",
        "aws_db_snapshot",
        "aws_db_subnet_group",
        "aws_rds_cluster",
        "aws_rds_cluster_endpoint",
        "aws_rds_cluster_instance",
        "aws_rds_cluster_parameter_group",
        "aws_rds_global_cluster",
        "aws_redshift_cluster",
        "aws_redshift_event_subscription",
        "aws_redshift_parameter_group",
        "aws_redshift_security_group",
        "aws_redshift_snapshot_copy_grant",
        "aws_redshift_snapshot_schedule",
        "aws_redshift_snapshot_schedule_association",
        "aws_redshift_subnet_group",
        "aws_resourcegroups_group",
        "aws_route53_delegation_set",
        "aws_route53_health_check",
        "aws_route53_query_log",
        "aws_route53_record",
        "aws_route53_zone",
        "aws_route53_zone_association",
        "aws_route53_resolver_endpoint",
        "aws_route53_resolver_rule",
        "aws_route53_resolver_rule_association",
        "aws_s3_access_point",
        "aws_s3_account_public_access_block",
        "aws_s3_bucket",
        "aws_s3_bucket_analytics_configuration",
        "aws_s3_bucket_inventory",
        "aws_s3_bucket_metric",
        "aws_s3_bucket_notification",
        "aws_s3_bucket_object",
        "aws_s3_bucket_policy",
        "aws_s3_bucket_public_access_block",
        "aws_ses_active_receipt_rule_set",
        "aws_ses_configuration_set",
        "aws_ses_domain_dkim",
        "aws_ses_domain_identity",
        "aws_ses_domain_identity_verification",
        "aws_ses_domain_mail_from",
        "aws_ses_email_identity",
        "aws_ses_event_destination",
        "aws_ses_identity_notification_topic",
        "aws_ses_identity_policy",
        "aws_ses_receipt_filter",
        "aws_ses_receipt_rule",
        "aws_ses_receipt_rule_set",
        "aws_ses_template",
        "aws_sns_platform_application",
        "aws_sns_sms_preferences",
        "aws_sns_topic",
        "aws_sns_topic_policy",
        "aws_sns_topic_subscription",
        "aws_sqs_queue",
        "aws_sqs_queue_policy",
        "aws_ssm_activation",
        "aws_ssm_association",
        "aws_ssm_document",
        "aws_ssm_maintenance_window",
        "aws_ssm_maintenance_window_target",
        "aws_ssm_maintenance_window_task",
        "aws_ssm_parameter",
        "aws_ssm_patch_baseline",
        "aws_ssm_patch_group",
        "aws_ssm_resource_data_sync",
        "aws_swf_domain",
        "aws_sagemaker_endpoint",
        "aws_sagemaker_endpoint_configuration",
        "aws_sagemaker_model",
        "aws_sagemaker_notebook_instance",
        "aws_sagemaker_notebook_instance_lifecycle_configuration",
        "aws_secretsmanager_secret",
        "aws_secretsmanager_secret_rotation",
        "aws_secretsmanager_secret_version",
        "aws_securityhub_account",
        "aws_securityhub_member",
        "aws_securityhub_product_subscription",
        "aws_securityhub_standards_subscription",
        "aws_servicecatalog_portfolio",
        "aws_service_discovery_http_namespace",
        "aws_service_discovery_private_dns_namespace",
        "aws_service_discovery_public_dns_namespace",
        "aws_service_discovery_service",
        "aws_servicequotas_service_quota",
        "aws_shield_protection",
        "aws_simpledb_domain",
        "aws_sfn_activity",
        "aws_sfn_state_machine",
        "aws_storagegateway_cache",
        "aws_storagegateway_cached_iscsi_volume",
        "aws_storagegateway_gateway",
        "aws_storagegateway_nfs_file_share",
        "aws_storagegateway_smb_file_share",
        "aws_storagegateway_upload_buffer",
        "aws_storagegateway_working_storage",
        "aws_transfer_server",
        "aws_transfer_ssh_key",
        "aws_transfer_user",
        "aws_customer_gateway",
        "aws_default_network_acl",
        "aws_default_route_table",
        "aws_default_security_group",
        "aws_default_subnet",
        "aws_default_vpc",
        "aws_default_vpc_dhcp_options",
        "aws_egress_only_internet_gateway",
        "aws_flow_log",
        "aws_internet_gateway",
        "aws_main_route_table_association",
        "aws_nat_gateway",
        "aws_network_acl",
        "aws_network_acl_rule",
        "aws_network_interface",
        "aws_network_interface_attachment",
        "aws_network_interface_sg_attachment",
        "aws_route",
        "aws_route_table",
        "aws_route_table_association",
        "aws_security_group",
        "aws_security_group_rule",
        "aws_subnet",
        "aws_vpc",
        "aws_vpc_dhcp_options",
        "aws_vpc_dhcp_options_association",
        "aws_vpc_endpoint",
        "aws_vpc_endpoint_connection_notification",
        "aws_vpc_endpoint_route_table_association",
        "aws_vpc_endpoint_service",
        "aws_vpc_endpoint_service_allowed_principal",
        "aws_vpc_endpoint_subnet_association",
        "aws_vpc_ipv4_cidr_block_association",
        "aws_vpc_peering_connection",
        "aws_vpc_peering_connection_accepter",
        "aws_vpc_peering_connection_options",
        "aws_vpn_connection",
        "aws_vpn_connection_route",
        "aws_vpn_gateway",
        "aws_vpn_gateway_attachment",
        "aws_vpn_gateway_route_propagation",
        "aws_waf_byte_match_set",
        "aws_waf_geo_match_set",
        "aws_waf_ipset",
        "aws_waf_rate_based_rule",
        "aws_waf_regex_match_set",
        "aws_waf_regex_pattern_set",
        "aws_waf_rule",
        "aws_waf_rule_group",
        "aws_waf_size_constraint_set",
        "aws_waf_sql_injection_match_set",
        "aws_waf_web_acl",
        "aws_waf_xss_match_set",
        "aws_wafregional_byte_match_set",
        "aws_wafregional_geo_match_set",
        "aws_wafregional_ipset",
        "aws_wafregional_rate_based_rule",
        "aws_wafregional_regex_match_set",
        "aws_wafregional_regex_pattern_set",
        "aws_wafregional_rule",
        "aws_wafregional_rule_group",
        "aws_wafregional_size_constraint_set",
        "aws_wafregional_sql_injection_match_set",
        "aws_wafregional_web_acl",
        "aws_wafregional_web_acl_association",
        "aws_wafregional_xss_match_set",
        "aws_wafv2_ip_set",
        "aws_wafv2_regex_pattern_set",
        "aws_wafv2_rule_group",
        "aws_wafv2_web_acl",
        "aws_wafv2_web_acl_association",
        "aws_wafv2_web_acl_logging_configuration",
        "aws_worklink_fleet",
        "aws_worklink_website_certificate_authority_association",
        "aws_workspaces_directory",
        "aws_workspaces_ip_group",
        "aws_workspaces_workspace",
        "aws_xray_sampling_rule",
        "aws_route53_vpc_association_authorization"
    ],
    "gcp": [
        "google_access_context_manager_access_level",
        "google_access_context_manager_access_policy",
        "google_access_context_manager_service_perimeter",
        "google_access_context_manager_service_perimeter_resource",
        "google_app_engine_application",
        "google_app_engine_application_url_dispatch_rules",
        "google_app_engine_domain_mapping",
        "google_app_engine_firewall_rule",
        "google_app_engine_flexible_app_version",
        "google_app_engine_service_split_traffic",
        "google_app_engine_standard_app_version",
        "google_bigquery_dataset",
        "google_bigquery_dataset_access",
        "google_bigquery_dataset_iam",
        "google_bigquery_job",
        "google_bigquery_table",
        "google_bigquery_data_transfer_config",
        "google_binary_authorization_attestor",
        "google_binary_authorization_attestor_iam",
        "google_binary_authorization_policy",
        "google_logging_billing_account_bucket_config",
        "google_logging_billing_account_exclusion",
        "google_logging_billing_account_sink",
        "google_logging_folder_bucket_config",
        "google_logging_folder_exclusion",
        "google_logging_folder_sink",
        "google_logging_metric",
        "google_logging_organization_bucket_config",
        "google_logging_organization_exclusion",
        "google_logging_organization_sink",
        "google_logging_project_bucket_config",
        "google_logging_project_exclusion",
        "google_logging_project_sink",
        "google_monitoring_alert_policy",
        "google_monitoring_dashboard",
        "google_monitoring_group",
        "google_monitoring_metric_descriptor",
        "google_monitoring_notification_channel",
        "google_monitoring_service",
        "google_monitoring_slo",
        "google_monitoring_uptime_check_config",
        "google_cloud_asset_folder_feed",
        "google_cloud_asset_organization_feed",
        "google_cloud_asset_project_feed",
        "google_bigtable_app_profile",
        "google_bigtable_gc_policy",
        "google_bigtable_instance",
        "google_bigtable_instance_iam",
        "google_bigtable_table",
        "google_cloudbuild_trigger",
        "google_composer_environment",
        "google_dns_managed_zone",
        "google_dns_policy",
        "google_dns_record_set",
        "google_deployment_manager_deployment",
        "google_endpoints_service",
        "google_endpoints_service_iam",
        "google_cloudfunctions_cloud_function_iam",
        "google_cloudfunctions_function",
        "google_healthcare_dataset",
        "google_healthcare_dataset_iam",
        "google_healthcare_dicom_store",
        "google_healthcare_dicom_store_iam",
        "google_healthcare_fhir_store",
        "google_healthcare_fhir_store_iam",
        "google_healthcare_hl7_v2_store",
        "google_healthcare_hl7_v2_store_iam",
        "google_cloudiot_device",
        "google_cloudiot_device_registry",
        "google_kms_crypto_key_iam",
        "google_kms_key_ring_iam",
        "google_kms_crypto_key",
        "google_kms_key_ring",
        "google_kms_key_ring_import_job",
        "google_kms_secret_ciphertext",
        "google_billing_account_iam_binding",
        "google_billing_account_iam_member",
        "google_billing_account_iam_policy",
        "google_folder",
        "google_folder_iam_audit_config",
        "google_folder_iam_binding",
        "google_folder_iam_member",
        "google_folder_iam_policy",
        "google_folder_organization_policy",
        "google_organization_iam_audit_config",
        "google_organization_iam_binding",
        "google_organization_iam_custom_role",
        "google_organization_iam_member",
        "google_organization_iam_policy",
        "google_organization_policy",
        "google_project",
        "google_project_iam_binding",
        "google_project_iam_member",
        "google_project_iam_custom_role",
        "google_project_organization_policy",
        "google_project_service",
        "google_service_account",
        "google_service_account_iam",
        "google_service_account_key",
        "google_usage_export_bucket",
        "google_pubsub_subscription",
        "google_pubsub_subscription_iam",
        "google_pubsub_topic",
        "google_pubsub_topic_iam",
        "google_cloud_run_domain_mapping",
        "google_cloud_run_service",
        "google_cloud_run_service_iam",
        "google_sql_database",
        "google_sql_database_instance",
        "google_sql_source_representation_instance",
        "google_sql_ssl_cert",
        "google_sql_user",
        "google_cloud_scheduler_job",
        "google_sourcerepo_repository",
        "google_sourcerepo_repository_iam",
        "google_spanner_database",
        "google_spanner_database_iam",
        "google_spanner_instance",
        "google_spanner_instance_iam",
        "google_storage_bucket",
        "google_storage_bucket_access_control",
        "google_storage_bucket_acl",
        "google_storage_bucket_iam_binding",
        "google_storage_bucket_iam_member",
        "google_storage_bucket_object",
        "google_storage_default_object_access_control",
        "google_storage_default_object_acl",
        "google_storage_hmac_key",
        "google_storage_notification",
        "google_storage_object_access_control",
        "google_storage_object_acl",
        "google_tpu_node",
        "google_cloud_tasks_queue",
        "google_compute_address",
        "google_compute_attached_disk",
        "google_compute_autoscaler",
        "google_compute_backend_bucket",
        "google_compute_backend_bucket_signed_url_key",
        "google_compute_backend_service",
        "google_compute_backend_service_signed_url_key",
        "google_compute_disk",
        "google_compute_disk_resource_policy_attachment",
        "google_compute_firewall",
        "google_compute_forwarding_rule",
        "google_compute_global_address",
        "google_compute_global_forwarding_rule",
        "google_compute_global_network_endpoint",
        "google_compute_global_network_endpoint_group",
        "google_compute_health_check",
        "google_compute_http_health_check",
        "google_compute_https_health_check",
        "google_compute_image",
        "google_compute_instance",
        "google_compute_instance_from_template",
        "google_compute_instance_group",
        "google_compute_instance_group_manager",
        "google_compute_instance_group_named_port",
        "google_compute_instance_iam",
        "google_compute_instance_template",
        "google_compute_interconnect_attachment",
        "google_compute_network",
        "google_compute_network_endpoint",
        "google_compute_network_endpoint_group",
        "google_compute_network_peering",
        "google_compute_network_peering_routes_config",
        "google_compute_node_group",
        "google_compute_node_template",
        "google_compute_project_default_network_tier",
        "google_compute_project_metadata",
        "google_compute_project_metadata_item",
        "google_compute_region_autoscaler",
        "google_compute_region_backend_service",
        "google_compute_region_disk",
        "google_compute_region_disk_resource_policy_attachment",
        "google_compute_region_health_check",
        "google_compute_region_instance_group_manager",
        "google_compute_region_ssl_certificate",
        "google_compute_region_target_http_proxy",
        "google_compute_region_target_https_proxy",
        "google_compute_region_url_map",
        "google_compute_reservation",
        "google_compute_resource_policy",
        "google_compute_route",
        "google_compute_router",
        "google_compute_router_bgp_peer",
        "google_compute_router_interface",
        "google_compute_router_nat",
        "google_compute_security_policy",
        "google_compute_shared_vpc_host_project",
        "google_compute_shared_vpc_service_project",
        "google_compute_snapshot",
        "google_compute_ssl_certificate",
        "google_compute_ssl_policy",
        "google_compute_subnetwork",
        "google_compute_subnetwork_iam",
        "google_compute_target_http_proxy",
        "google_compute_target_https_proxy",
        "google_compute_target_instance",
        "google_compute_target_pool",
        "google_compute_target_ssl_proxy",
        "google_compute_target_tcp_proxy",
        "google_compute_url_map",
        "google_compute_vpn_gateway",
        "google_compute_vpn_tunnel",
        "google_container_analysis_note",
        "google_container_analysis_occurrence",
        "google_container_registry",
        "google_data_catalog_entry",
        "google_data_catalog_entry_group",
        "google_data_catalog_entry_group_iam",
        "google_data_catalog_tag",
        "google_data_catalog_tag_template",
        "google_dataflow_job",
        "google_dataproc_autoscaling_policy",
        "google_dataproc_cluster",
        "google_dataproc_cluster_iam",
        "google_dataproc_job",
        "google_dataproc_job_iam",
        "google_datastore_index",
        "google_dialogflow_agent",
        "google_dialogflow_entity_type",
        "google_dialogflow_intent",
        "google_filestore_instance",
        "google_firestore_index",
        "google_identity_platform_default_supported_idp_config",
        "google_identity_platform_inbound_saml_config",
        "google_identity_platform_oauth_idp_config",
        "google_identity_platform_tenant",
        "google_identity_platform_tenant_default_supported_idp_config",
        "google_identity_platform_tenant_inbound_saml_config",
        "google_identity_platform_tenant_oauth_idp_config",
        "google_iap_app_engine_service_iam",
        "google_iap_app_engine_version_iam",
        "google_iap_brand",
        "google_iap_client",
        "google_iap_tunnel_instance_iam",
        "google_iap_web_backend_service_iam",
        "google_iap_web_iam",
        "google_iap_web_type_app_engine_iam",
        "google_iap_web_type_compute_iam",
        "google_container_cluster",
        "google_container_node_pool",
        "google_ml_engine_model",
        "google_redis_instance",
        "google_network_management_connectivity_test_resource",
        "google_os_config_patch_deployment",
        "google_os_login_ssh_public_key",
        "google_resource_manager_lien",
        "google_runtimeconfig_config",
        "google_runtimeconfig_config_iam",
        "google_runtimeconfig_variable",
        "google_secret_manager_secret",
        "google_secret_manager_secret_iam",
        "google_secret_manager_secret_version",
        "google_scc_source",
        "google_vpc_access_connector",
        "google_service_networking_connection",
        "google_storage_transfer_job",
        "google_project_iam",
        "google_storage_bucket_iam",
        "google_dataflow_flex_template_job",
        "google_active_directory_domain"
    ],
    "azure": [
        "azurerm_api_management",
        "azurerm_api_management_api",
        "azurerm_api_management_api_operation",
        "azurerm_api_management_api_operation_policy",
        "azurerm_api_management_api_policy",
        "azurerm_api_management_api_schema",
        "azurerm_api_management_api_version_set",
        "azurerm_api_management_authorization_server",
        "azurerm_api_management_backend",
        "azurerm_api_management_certificate",
        "azurerm_api_management_diagnostic",
        "azurerm_api_management_group",
        "azurerm_api_management_group_user",
        "azurerm_api_management_identity_provider_aad",
        "azurerm_api_management_identity_provider_facebook",
        "azurerm_api_management_identity_provider_google",
        "azurerm_api_management_identity_provider_microsoft",
        "azurerm_api_management_identity_provider_twitter",
        "azurerm_api_management_logger",
        "azurerm_api_management_named_value",
        "azurerm_api_management_openid_connect_provider",
        "azurerm_api_management_product",
        "azurerm_api_management_product_api",
        "azurerm_api_management_product_group",
        "azurerm_api_management_product_policy",
        "azurerm_api_management_property",
        "azurerm_api_management_subscription",
        "azurerm_api_management_user",
        "azurerm_analysis_services_server",
        "azurerm_app_configuration",
        "azurerm_app_service",
        "azurerm_app_service_active_slot",
        "azurerm_app_service_certificate",
        "azurerm_app_service_certificate_order",
        "azurerm_app_service_custom_hostname_binding",
        "azurerm_app_service_environment",
        "azurerm_app_service_hybrid_connection",
        "azurerm_app_service_plan",
        "azurerm_app_service_slot",
        "azurerm_app_service_slot_virtual_network_swift_connection",
        "azurerm_app_service_source_control_token",
        "azurerm_app_service_virtual_network_swift_connection",
        "azurerm_function_app",
        "azurerm_function_app_slot",
        "azurerm_application_insights",
        "azurerm_application_insights_analytics_item",
        "azurerm_application_insights_api_key",
        "azurerm_application_insights_web_test",
        "azurerm_role_assignment",
        "azurerm_role_definition",
        "azurerm_user_assigned_identity",
        "azurerm_automation_account",
        "azurerm_automation_certificate",
        "azurerm_automation_connection",
        "azurerm_automation_connection_certificate",
        "azurerm_automation_connection_classic_certificate",
        "azurerm_automation_connection_service_principal",
        "azurerm_automation_credential",
        "azurerm_automation_dsc_configuration",
        "azurerm_automation_dsc_nodeconfiguration",
        "azurerm_automation_job_schedule",
        "azurerm_automation_module",
        "azurerm_automation_runbook",
        "azurerm_automation_schedule",
        "azurerm_automation_variable_bool",
        "azurerm_automation_variable_datetime",
        "azurerm_automation_variable_int",
        "azurerm_automation_variable_string",
        "azurerm_resource_group",
        "azurerm_batch_account",
        "azurerm_batch_application",
        "azurerm_batch_certificate",
        "azurerm_batch_pool",
        "azurerm_blueprint_assignment",
        "azurerm_bot_channel_directline",
        "azurerm_bot_channel_email",
        "azurerm_bot_channel_ms_teams",
        "azurerm_bot_channel_slack",
        "azurerm_bot_channels_registration",
        "azurerm_bot_connection",
        "azurerm_bot_web_app",
        "azurerm_cdn_endpoint",
        "azurerm_cdn_profile",
        "azurerm_cognitive_account",
        "azurerm_availability_set",
        "azurerm_dedicated_host",
        "azurerm_dedicated_host_group",
        "azurerm_disk_encryption_set",
        "azurerm_image",
        "azurerm_linux_virtual_machine",
        "azurerm_linux_virtual_machine_scale_set",
        "azurerm_managed_disk",
        "azurerm_marketplace_agreement",
        "azurerm_orchestrated_virtual_machine_scale_set",
        "azurerm_proximity_placement_group",
        "azurerm_shared_image",
        "azurerm_shared_image_gallery",
        "azurerm_shared_image_version",
        "azurerm_snapshot",
        "azurerm_virtual_machine",
        "azurerm_virtual_machine_data_disk_attachment",
        "azurerm_virtual_machine_extension",
        "azurerm_virtual_machine_scale_set",
        "azurerm_virtual_machine_scale_set_extension",
        "azurerm_windows_virtual_machine",
        "azurerm_windows_virtual_machine_scale_set",
        "azurerm_container_group",
        "azurerm_container_registry",
        "azurerm_container_registry_webhook",
        "azurerm_kubernetes_cluster",
        "azurerm_kubernetes_cluster_node_pool",
        "azurerm_cosmosdb_account",
        "azurerm_cosmosdb_cassandra_keyspace",
        "azurerm_cosmosdb_gremlin_database",
        "azurerm_cosmosdb_gremlin_graph",
        "azurerm_cosmosdb_mongo_collection",
        "azurerm_cosmosdb_mongo_database",
        "azurerm_cosmosdb_sql_container",
        "azurerm_cosmosdb_sql_database",
        "azurerm_cosmosdb_table",
        "azurerm_cost_management_export_resource_group",
        "azurerm_custom_provider",
        "azurerm_dns_a_record",
        "azurerm_dns_aaaa_record",
        "azurerm_dns_caa_record",
        "azurerm_dns_cname_record",
        "azurerm_dns_mx_record",
        "azurerm_dns_ns_record",
        "azurerm_dns_ptr_record",
        "azurerm_dns_srv_record",
        "azurerm_dns_txt_record",
        "azurerm_dns_zone",
        "azurerm_kusto_attached_database_configuration",
        "azurerm_kusto_cluster",
        "azurerm_kusto_cluster_customer_managed_key",
        "azurerm_kusto_cluster_principal_assignment",
        "azurerm_kusto_database",
        "azurerm_kusto_database_principal",
        "azurerm_kusto_database_principal_assignment",
        "azurerm_kusto_eventhub_data_connection",
        "azurerm_data_factory",
        "azurerm_data_factory_dataset_azure_blob",
        "azurerm_data_factory_dataset_cosmosdb_sqlapi",
        "azurerm_data_factory_dataset_delimited_text",
        "azurerm_data_factory_dataset_http",
        "azurerm_data_factory_dataset_json",
        "azurerm_data_factory_dataset_mysql",
        "azurerm_data_factory_dataset_postgresql",
        "azurerm_data_factory_dataset_sql_server_table",
        "azurerm_data_factory_integration_runtime_managed",
        "azurerm_data_factory_integration_runtime_self_hosted",
        "azurerm_data_factory_linked_service_azure_blob_storage",
        "azurerm_data_factory_linked_service_azure_file_storage",
        "azurerm_data_factory_linked_service_azure_function",
        "azurerm_data_factory_linked_service_cosmosdb",
        "azurerm_data_factory_linked_service_data_lake_storage_gen2",
        "azurerm_data_factory_linked_service_key_vault",
        "azurerm_data_factory_linked_service_mysql",
        "azurerm_data_factory_linked_service_postgresql",
        "azurerm_data_factory_linked_service_sftp",
        "azurerm_data_factory_linked_service_sql_server",
        "azurerm_data_factory_linked_service_web",
        "azurerm_data_factory_pipeline",
        "azurerm_data_factory_trigger_schedule",
        "azurerm_data_lake_analytics_account",
        "azurerm_data_lake_analytics_firewall_rule",
        "azurerm_data_lake_store",
        "azurerm_data_lake_store_file",
        "azurerm_data_lake_store_firewall_rule",
        "azurerm_data_share",
        "azurerm_data_share_account",
        "azurerm_data_share_dataset_blob_storage",
        "azurerm_data_share_dataset_data_lake_gen1",
        "azurerm_mariadb_configuration",
        "azurerm_mariadb_database",
        "azurerm_mariadb_firewall_rule",
        "azurerm_mariadb_server",
        "azurerm_mariadb_virtual_network_rule",
        "azurerm_mssql_database",
        "azurerm_mssql_database_vulnerability_assessment_rule_baseline",
        "azurerm_mssql_elasticpool",
        "azurerm_mssql_server",
        "azurerm_mssql_server_security_alert_policy",
        "azurerm_mssql_server_vulnerability_assessment",
        "azurerm_mssql_virtual_machine",
        "azurerm_mysql_active_directory_administrator",
        "azurerm_mysql_configuration",
        "azurerm_mysql_database",
        "azurerm_mysql_firewall_rule",
        "azurerm_mysql_server",
        "azurerm_mysql_virtual_network_rule",
        "azurerm_postgresql_active_directory_administrator",
        "azurerm_postgresql_configuration",
        "azurerm_postgresql_database",
        "azurerm_postgresql_firewall_rule",
        "azurerm_postgresql_server",
        "azurerm_postgresql_virtual_network_rule",
        "azurerm_sql_active_directory_administrator",
        "azurerm_sql_database",
        "azurerm_sql_elasticpool",
        "azurerm_sql_failover_group",
        "azurerm_sql_firewall_rule",
        "azurerm_sql_server",
        "azurerm_sql_virtual_network_rule",
        "azurerm_database_migration_project",
        "azurerm_database_migration_service",
        "azurerm_databricks_workspace",
        "azurerm_dev_test_global_vm_shutdown_schedule",
        "azurerm_dev_test_lab",
        "azurerm_dev_test_linux_virtual_machine",
        "azurerm_dev_test_policy",
        "azurerm_dev_test_schedule",
        "azurerm_dev_test_virtual_network",
        "azurerm_dev_test_windows_virtual_machine",
        "azurerm_devspace_controller",
        "azurerm_hdinsight_hadoop_cluster",
        "azurerm_hdinsight_hbase_cluster",
        "azurerm_hdinsight_interactive_query_cluster",
        "azurerm_hdinsight_kafka_cluster",
        "azurerm_hdinsight_ml_services_cluster",
        "azurerm_hdinsight_rserver_cluster",
        "azurerm_hdinsight_spark_cluster",
        "azurerm_hdinsight_storm_cluster",
        "azurerm_dedicated_hardware_security_module",
        "azurerm_healthcare_service",
        "azurerm_iotcentral_application",
        "azurerm_iothub",
        "azurerm_iothub_consumer_group",
        "azurerm_iothub_dps",
        "azurerm_iothub_dps_certificate",
        "azurerm_iothub_dps_shared_access_policy",
        "azurerm_iothub_shared_access_policy",
        "azurerm_key_vault",
        "azurerm_key_vault_access_policy",
        "azurerm_key_vault_certificate",
        "azurerm_key_vault_certificate_issuer",
        "azurerm_key_vault_key",
        "azurerm_key_vault_secret",
        "azurerm_lb",
        "azurerm_lb_backend_address_pool",
        "azurerm_lb_nat_pool",
        "azurerm_lb_nat_rule",
        "azurerm_lb_outbound_rule",
        "azurerm_lb_probe",
        "azurerm_lb_rule",
        "azurerm_log_analytics_datasource_windows_event",
        "azurerm_log_analytics_datasource_windows_performance_counter",
        "azurerm_log_analytics_linked_service",
        "azurerm_log_analytics_solution",
        "azurerm_log_analytics_workspace",
        "azurerm_logic_app_action_custom",
        "azurerm_logic_app_action_http",
        "azurerm_logic_app_integration_account",
        "azurerm_logic_app_trigger_custom",
        "azurerm_logic_app_trigger_http_request",
        "azurerm_logic_app_trigger_recurrence",
        "azurerm_logic_app_workflow",
        "azurerm_machine_learning_workspace",
        "azurerm_maintenance_assignment_dedicated_host",
        "azurerm_maintenance_assignment_virtual_machine",
        "azurerm_maintenance_configuration",
        "azurerm_managed_application",
        "azurerm_managed_application_definition",
        "azurerm_management_group",
        "azurerm_management_lock",
        "azurerm_maps_account",
        "azurerm_media_services_account",
        "azurerm_eventgrid_domain",
        "azurerm_eventgrid_domain_topic",
        "azurerm_eventgrid_event_subscription",
        "azurerm_eventgrid_topic",
        "azurerm_eventhub",
        "azurerm_eventhub_authorization_rule",
        "azurerm_eventhub_cluster",
        "azurerm_eventhub_consumer_group",
        "azurerm_eventhub_namespace",
        "azurerm_eventhub_namespace_authorization_rule",
        "azurerm_eventhub_namespace_disaster_recovery_config",
        "azurerm_iothub_endpoint_eventhub",
        "azurerm_iothub_endpoint_servicebus_queue",
        "azurerm_iothub_endpoint_servicebus_topic",
        "azurerm_iothub_endpoint_storage_container",
        "azurerm_iothub_fallback_route",
        "azurerm_iothub_route",
        "azurerm_notification_hub",
        "azurerm_notification_hub_authorization_rule",
        "azurerm_notification_hub_namespace",
        "azurerm_relay_hybrid_connection",
        "azurerm_relay_namespace",
        "azurerm_servicebus_namespace",
        "azurerm_servicebus_namespace_authorization_rule",
        "azurerm_servicebus_namespace_network_rule_set",
        "azurerm_servicebus_queue",
        "azurerm_servicebus_queue_authorization_rule",
        "azurerm_servicebus_subscription",
        "azurerm_servicebus_subscription_rule",
        "azurerm_servicebus_topic",
        "azurerm_servicebus_topic_authorization_rule",
        "azurerm_signalr_service",
        "azurerm_spatial_anchors_account",
        "azurerm_monitor_action_group",
        "azurerm_monitor_action_rule_action_group",
        "azurerm_monitor_action_rule_suppression",
        "azurerm_monitor_activity_log_alert",
        "azurerm_monitor_autoscale_setting",
        "azurerm_monitor_diagnostic_setting",
        "azurerm_monitor_log_profile",
        "azurerm_monitor_metric_alert",
        "azurerm_monitor_scheduled_query_rules_alert",
        "azurerm_monitor_scheduled_query_rules_log",
        "azurerm_netapp_account",
        "azurerm_netapp_pool",
        "azurerm_netapp_snapshot",
        "azurerm_netapp_volume",
        "azurerm_application_gateway",
        "azurerm_application_security_group",
        "azurerm_bastion_host",
        "azurerm_express_route_circuit",
        "azurerm_express_route_circuit_authorization",
        "azurerm_express_route_circuit_peering",
        "azurerm_express_route_gateway",
        "azurerm_firewall",
        "azurerm_firewall_application_rule_collection",
        "azurerm_firewall_nat_rule_collection",
        "azurerm_firewall_network_rule_collection",
        "azurerm_frontdoor",
        "azurerm_frontdoor_custom_https_configuration",
        "azurerm_frontdoor_firewall_policy",
        "azurerm_local_network_gateway",
        "azurerm_nat_gateway",
        "azurerm_nat_gateway_public_ip_association",
        "azurerm_network_ddos_protection_plan",
        "azurerm_network_interface",
        "azurerm_network_interface_application_gateway_backend_address_pool_association",
        "azurerm_network_interface_application_security_group_association",
        "azurerm_network_interface_backend_address_pool_association",
        "azurerm_network_interface_nat_rule_association",
        "azurerm_network_interface_security_group_association",
        "azurerm_network_packet_capture",
        "azurerm_network_profile",
        "azurerm_network_security_group",
        "azurerm_network_security_rule",
        "azurerm_network_watcher",
        "azurerm_network_watcher_flow_log",
        "azurerm_packet_capture",
        "azurerm_point_to_site_vpn_gateway",
        "azurerm_private_endpoint",
        "azurerm_private_link_service",
        "azurerm_public_ip",
        "azurerm_public_ip_prefix",
        "azurerm_route",
        "azurerm_route_filter",
        "azurerm_route_table",
        "azurerm_subnet",
        "azurerm_subnet_nat_gateway_association",
        "azurerm_subnet_network_security_group_association",
        "azurerm_subnet_route_table_association",
        "azurerm_traffic_manager_endpoint",
        "azurerm_traffic_manager_profile",
        "azurerm_virtual_hub",
        "azurerm_virtual_hub_connection",
        "azurerm_virtual_network",
        "azurerm_virtual_network_gateway",
        "azurerm_virtual_network_gateway_connection",
        "azurerm_virtual_network_peering",
        "azurerm_virtual_wan",
        "azurerm_vpn_gateway",
        "azurerm_vpn_server_configuration",
        "azurerm_web_application_firewall_policy",
        "azurerm_policy_assignment",
        "azurerm_policy_definition",
        "azurerm_policy_remediation",
        "azurerm_policy_set_definition",
        "azurerm_dashboard",
        "azurerm_powerbi_embedded",
        "azurerm_private_dns_a_record",
        "azurerm_private_dns_aaaa_record",
        "azurerm_private_dns_cname_record",
        "azurerm_private_dns_mx_record",
        "azurerm_private_dns_ptr_record",
        "azurerm_private_dns_srv_record",
        "azurerm_private_dns_txt_record",
        "azurerm_private_dns_zone",
        "azurerm_private_dns_zone_virtual_network_link",
        "azurerm_backup_container_storage_account",
        "azurerm_backup_policy_file_share",
        "azurerm_backup_policy_vm",
        "azurerm_backup_protected_file_share",
        "azurerm_backup_protected_vm",
        "azurerm_recovery_services_vault",
        "azurerm_site_recovery_fabric",
        "azurerm_site_recovery_network_mapping",
        "azurerm_site_recovery_protection_container",
        "azurerm_site_recovery_protection_container_mapping",
        "azurerm_site_recovery_replicated_vm",
        "azurerm_site_recovery_replication_policy",
        "azurerm_redis_cache",
        "azurerm_redis_firewall_rule",
        "azurerm_search_service",
        "azurerm_advanced_threat_protection",
        "azurerm_security_center_contact",
        "azurerm_security_center_subscription_pricing",
        "azurerm_security_center_workspace",
        "azurerm_sentinel_alert_rule_ms_security_incident",
        "azurerm_sentinel_alert_rule_scheduled",
        "azurerm_service_fabric_cluster",
        "azurerm_spring_cloud_app",
        "azurerm_spring_cloud_service",
        "azurerm_hpc_cache",
        "azurerm_hpc_cache_blob_target",
        "azurerm_hpc_cache_nfs_target",
        "azurerm_storage_account",
        "azurerm_storage_account_customer_managed_key",
        "azurerm_storage_account_network_rules",
        "azurerm_storage_blob",
        "azurerm_storage_container",
        "azurerm_storage_data_lake_gen2_filesystem",
        "azurerm_storage_management_policy",
        "azurerm_storage_queue",
        "azurerm_storage_share",
        "azurerm_storage_share_directory",
        "azurerm_storage_table",
        "azurerm_storage_table_entity",
        "azurerm_stream_analytics_function_javascript_udf",
        "azurerm_stream_analytics_job",
        "azurerm_stream_analytics_output_blob",
        "azurerm_stream_analytics_output_eventhub",
        "azurerm_stream_analytics_output_mssql",
        "azurerm_stream_analytics_output_servicebus_queue",
        "azurerm_stream_analytics_output_servicebus_topic",
        "azurerm_stream_analytics_reference_input_blob",
        "azurerm_stream_analytics_stream_input_blob",
        "azurerm_stream_analytics_stream_input_eventhub",
        "azurerm_stream_analytics_stream_input_iothub",
        "azurerm_synapse_workspace",
        "azurerm_template_deployment",
        "azurerm_iot_time_series_insights_access_policy",
        "azurerm_iot_time_series_insights_reference_data_set",
        "azurerm_iot_time_series_insights_standard_environment"
    ]
}



from __future__ import annotations

import logging
import os
from typing import Dict, Any, TYPE_CHECKING

import dpath

from checkov.common.models.enums import CheckResult
from checkov.common.util.consts import LINE_FIELD_NAMES, START_LINE, END_LINE
from checkov.runner_filter import RunnerFilter
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration
from checkov.common.models.consts import YAML_COMMENT_MARK
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import filter_ignored_paths
from checkov.common.util.type_forcers import force_list
from checkov.kubernetes.parser.parser import parse

if TYPE_CHECKING:
    from checkov.common.typing import _SkippedCheck, _CheckResult, _EntityContext

EXCLUDED_FILE_NAMES = {"package.json", "package-lock.json"}
K8_POSSIBLE_ENDINGS = {".yaml", ".yml", ".json"}
DEFAULT_NESTED_RESOURCE_TYPE = "Pod"
SUPPORTED_POD_CONTAINERS_TYPES = {"Deployment", "DeploymentConfig", "DaemonSet", "Job", "ReplicaSet", "ReplicationController", "StatefulSet"}
PARENT_RESOURCE_KEY_NAME = "_parent_resource"
PARENT_RESOURCE_ID_KEY_NAME = "_parent_resource_id"
FILTERED_RESOURCES_FOR_EDGE_BUILDERS = ["NetworkPolicy"]


def get_folder_definitions(
        root_folder: str, excluded_paths: list[str] | None
) -> tuple[dict[str, list[dict[str, Any]]], dict[str, list[tuple[int, str]]]]:
    files_list = []
    for root, d_names, f_names in os.walk(root_folder):
        filter_ignored_paths(root, d_names, excluded_paths)
        filter_ignored_paths(root, f_names, excluded_paths)

        for file in f_names:
            file_ending = os.path.splitext(file)[1]
            if file_ending in K8_POSSIBLE_ENDINGS:
                full_path = os.path.join(root, file)
                if "/." not in full_path and file not in EXCLUDED_FILE_NAMES:
                    # skip temp directories
                    files_list.append(full_path)
    return get_files_definitions(files_list)


def get_files_definitions(files: list[str]) -> tuple[dict[str, list[dict[str, Any]]], dict[str, list[tuple[int, str]]]]:
    definitions = {}
    definitions_raw = {}
    results = parallel_runner.run_function(_parse_file, files)
    for result in results:
        if result:
            path, parse_result = result
            if parse_result:
                definitions[path], definitions_raw[path] = parse_result
    return definitions, definitions_raw


def _parse_file(filename: str) -> tuple[str, tuple[list[dict[str, Any]], list[tuple[int, str]]] | None] | None:
    try:
        return filename, parse(filename)
    except (TypeError, ValueError):
        logging.warning(f"Kubernetes skipping {filename} as it is not a valid Kubernetes template", exc_info=True)

    return None


def get_skipped_checks(entity_conf: dict[str, Any]) -> list[_SkippedCheck]:
    skipped = []
    metadata = {}
    bc_id_mapping = metadata_integration.bc_to_ckv_id_mapping
    if not isinstance(entity_conf, dict):
        return skipped
    if "metadata" in entity_conf.keys():
        metadata = entity_conf["metadata"]
    if metadata and "annotations" in metadata.keys() and metadata["annotations"] is not None:
        if isinstance(metadata["annotations"], dict):
            metadata["annotations"] = force_list(metadata["annotations"])
        for annotation in metadata["annotations"]:
            if not isinstance(annotation, dict):
                logging.debug(f"Parse of Annotation Failed for {annotation}: {entity_conf}")
                continue
            for key in annotation:
                skipped_item: "_SkippedCheck" = {}
                if "checkov.io/skip" in key or "bridgecrew.io/skip" in key:
                    if "=" in annotation[key]:
                        (skipped_item["id"], skipped_item["suppress_comment"]) = annotation[key].split("=")
                    else:
                        skipped_item["id"] = annotation[key]
                        skipped_item["suppress_comment"] = "No comment provided"

                    # No matter which ID was used to skip, save the pair of IDs in the appropriate fields
                    if bc_id_mapping and skipped_item["id"] in bc_id_mapping:
                        skipped_item["bc_id"] = skipped_item["id"]
                        skipped_item["id"] = bc_id_mapping[skipped_item["id"]]
                    elif metadata_integration.check_metadata:
                        skipped_item["bc_id"] = metadata_integration.get_bc_id(skipped_item["id"])
                    skipped.append(skipped_item)
    return skipped


def create_definitions(
    root_folder: str | None,
    files: list[str] | None = None,
    runner_filter: RunnerFilter | None = None,
) -> tuple[dict[str, list[dict[str, Any]]], dict[str, list[tuple[int, str]]]]:
    runner_filter = runner_filter or RunnerFilter()
    definitions: dict[str, list[dict[str, Any]]] = {}
    definitions_raw: dict[str, list[tuple[int, str]]] = {}
    if files:
        definitions, definitions_raw = get_files_definitions(files)

    if root_folder:
        definitions, definitions_raw = get_folder_definitions(root_folder, runner_filter.excluded_paths)

    return definitions, definitions_raw


def build_definitions_context(
    definitions: dict[str, list[dict[str, Any]]], definitions_raw: dict[str, list[tuple[int, str]]]
) -> dict[str, dict[str, Any]]:
    definitions_context: Dict[str, Dict[str, Any]] = {}
    # iterate on the files
    for file_path, resources in definitions.items():
        for resource in resources:
            if resource.get("kind") == "List":
                # this could be inefficient, if more than one 'List' object exists in the same file
                resources = resources[:]
                resources.extend(item for item in resource.get("items", []) if item)
                resources.remove(resource)

        # iterate on the resources
        for resource in resources:
            if is_invalid_k8_definition(resource):
                continue
            resource_id = get_resource_id(resource)
            if not resource_id:
                continue
            start_line = resource[START_LINE]
            end_line = min(resource[END_LINE], len(definitions_raw[file_path]))
            first_line_index = 0
            # skip empty lines
            while not str.strip(definitions_raw[file_path][first_line_index][1]):
                first_line_index += 1
            # check if the file is a json file
            if str.strip(definitions_raw[file_path][first_line_index][1])[0] == "{":
                start_line += 1
                end_line += 1
            else:
                # add resource comments to definition lines
                current_line = str.strip(definitions_raw[file_path][start_line - 1][1])
                while not current_line or current_line[0] == YAML_COMMENT_MARK:
                    start_line -= 1
                    current_line = str.strip(definitions_raw[file_path][start_line - 1][1])

                # remove next resource comments from definition lines
                current_line = str.strip(definitions_raw[file_path][end_line - 1][1])
                while not current_line or current_line[0] == YAML_COMMENT_MARK:
                    end_line -= 1
                    current_line = str.strip(definitions_raw[file_path][end_line - 1][1])

            code_lines = definitions_raw[file_path][start_line - 1: end_line]
            dpath.new(
                definitions_context,
                [file_path, resource_id],
                {"start_line": start_line, "end_line": end_line, "code_lines": code_lines},
            )

            skipped_checks = get_skipped_checks(resource)
            dpath.new(
                definitions_context,
                [file_path, resource_id, "skipped_checks"],
                skipped_checks,
            )
    return definitions_context


def is_invalid_k8_definition(definition: Dict[str, Any]) -> bool:
    return (
        not isinstance(definition, dict)
        or 'apiVersion' not in definition.keys()
        or 'kind' not in definition.keys()
        or isinstance(definition.get("kind"), int)
        or not isinstance(definition.get('metadata'), dict)
    )


def is_invalid_k8_pod_definition(definition: Dict[str, Any]) -> bool:
    if not isinstance(definition, dict):
        return True
    metadata = definition.get('metadata')
    if not isinstance(metadata, dict):
        return True
    spec = definition.get('spec')
    if not isinstance(spec, dict) and not isinstance(spec, list):
        return True
    labels = metadata.get('labels')
    name = metadata.get('name')
    if name is None and labels is None:
        return True
    return False


def get_resource_id(resource: dict[str, Any] | None) -> str | None:
    if not resource:
        return None

    resource_type = resource.get("kind", DEFAULT_NESTED_RESOURCE_TYPE)
    metadata = resource.get("metadata") or {}
    namespace = metadata.get("namespace", "default")
    name = metadata.get("name")
    if name:
        return f'{resource_type}.{namespace}.{name}'
    labels = metadata.get("labels")
    if labels:
        return build_resource_id_from_labels(resource_type, namespace, labels, resource)
    return None


def build_resource_id_from_labels(resource_type: str,
                                  namespace: str,
                                  labels: dict[str, str],
                                  resource: dict[str, Any]) -> str:
    labels_list = [
        f"{label}-{value}"
        for label, value in labels.items()
        if label not in LINE_FIELD_NAMES
    ]
    labels_string = ".".join(labels_list) if labels_list else "default"
    parent_resource = resource.get(PARENT_RESOURCE_KEY_NAME)
    if parent_resource:
        resource_id = f'{resource_type}.{namespace}.{parent_resource}.{labels_string}'
    else:
        resource_id = f'{resource_type}.{namespace}.{labels_string}'
    return resource_id


def remove_metadata_from_attribute(attribute: dict[str, Any] | None) -> None:
    if isinstance(attribute, dict):
        attribute.pop("__startline__", None)
        attribute.pop("__endline__", None)


def create_check_result(check_result: _CheckResult, entity_context: _EntityContext, check_id: str) -> _CheckResult:
    """Creates a cleaned version of check_result for further usage"""

    clean_check_result: _CheckResult = {
        "result": check_result["result"],
        "evaluated_keys": check_result["evaluated_keys"],
    }

    for skipped_check in entity_context.get("skipped_checks", []):
        if skipped_check["id"] == check_id:
            clean_check_result["result"] = CheckResult.SKIPPED
            clean_check_result["suppress_comment"] = skipped_check["suppress_comment"]
            break

    return clean_check_result



from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class EKSPublicAccess(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Amazon EKS public endpoint disabled"
        id = "CKV_AWS_39"
        supported_resources = ("aws_eks_cluster",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "vpc_config/[0]/endpoint_public_access"

    def get_expected_value(self) -> Any:
        return False


check = EKSPublicAccess()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class GuarddutyDetectorEnabled(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure that Guard Duty detector is enabled"
        id = "CKV_AWS_238"
        supported_resources = ['aws_guardduty_detector']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return 'enable'


check = GuarddutyDetectorEnabled()



from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class APIGatewayMethodSettingsDataTrace(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure Data Trace is not enabled in API Gateway Method Settings"
        id = "CKV_AWS_276"
        supported_resources = ('aws_api_gateway_method_settings',)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "settings/[0]/data_trace_enabled"

    def get_forbidden_values(self):
        return [True]


check = APIGatewayMethodSettingsDataTrace()



import json
import re

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from policyuniverse.policy import Policy
from typing import List

DATA_TO_JSON_PATTERN = re.compile(r"\$?\{?(.+?)(?=.json).json\}?")


class GlacierVaultAnyPrincipal(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Glacier Vault access policy is not public by only allowing specific services or principals to access it"
        id = "CKV_AWS_167"
        supported_resources = ['aws_glacier_vault']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'access_policy' not in conf:
            return CheckResult.PASSED
        policy_obj = conf['access_policy'][0]
        if isinstance(policy_obj, str):
            if re.match(DATA_TO_JSON_PATTERN, policy_obj):
                return CheckResult.UNKNOWN
            else:
                try:
                    policy_obj = json.loads(policy_obj)
                except Exception:
                    return CheckResult.UNKNOWN
        try:
            policy = Policy(policy_obj)
        except TypeError:
            return CheckResult.UNKNOWN
        if policy.is_internet_accessible():
            return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['access_policy']


check = GlacierVaultAnyPrincipal()



import datetime
import json
from copy import deepcopy
from json import JSONDecodeError
from typing import Any, Dict

from lark import Tree
from bc_jsonpath_ng import parse, JSONPath

from checkov.common.bridgecrew.severities import Severity
from checkov.common.output.common import ImageDetails
from checkov.common.packaging.version import LegacyVersion, Version
from detect_secrets.core.potential_secret import PotentialSecret

type_of_function = type(lambda x: x)


class CustomJSONEncoder(json.JSONEncoder):
    def default(self, o: Any) -> Any:
        from checkov.terraform.modules.module_objects import TFModule, TFDefinitionKey
        if isinstance(o, set):
            return list(o)
        elif isinstance(o, Tree):
            return str(o)
        elif isinstance(o, datetime.date):
            return str(o)
        elif isinstance(o, (Version, LegacyVersion)):
            return str(o)
        elif isinstance(o, Severity):
            return o.name
        elif isinstance(o, complex):
            return str(o)
        elif isinstance(o, ImageDetails):
            return o.__dict__
        elif isinstance(o, type_of_function):
            return str(o)
        elif isinstance(o, TFDefinitionKey):
            return str(o)
        elif isinstance(o, TFModule):
            return dict(o)
        elif isinstance(o, PotentialSecret):
            return o.json()
        else:
            return json.JSONEncoder.default(self, o)

    def encode(self, obj: Any) -> str:
        return super().encode(self._encode(obj))

    def _encode(self, obj: Any) -> Any:
        if isinstance(obj, dict):
            return {self.encode_key(k): v for k, v in obj.items()}
        else:
            return obj

    @staticmethod
    def encode_key(key: Any) -> Any:
        from checkov.terraform.modules.module_objects import TFModule, TFDefinitionKey
        if isinstance(key, TFDefinitionKey):
            return str(key)
        if isinstance(key, TFModule):
            return str(key)
        else:
            return key


def object_hook(dct: Dict[Any, Any]) -> Any:
    from checkov.terraform.modules.module_objects import TFModule, TFDefinitionKey
    from checkov.common.util.consts import RESOLVED_MODULE_ENTRY_NAME
    try:
        if dct is None:
            return None
        if isinstance(dct, dict):
            dct_obj = deepcopy(dct)
            if 'tf_source_modules' in dct and 'file_path' in dct:
                return TFDefinitionKey(file_path=dct["file_path"],
                                       tf_source_modules=object_hook(dct["tf_source_modules"]))
            if 'path' in dct and 'name' in dct and 'foreach_idx' in dct and 'nested_tf_module' in dct:
                return TFModule(path=dct['path'], name=dct['name'], foreach_idx=dct['foreach_idx'],
                                nested_tf_module=object_hook(dct['nested_tf_module']))
            for key, value in dct.items():
                if key == RESOLVED_MODULE_ENTRY_NAME:
                    resolved_classes = []
                    for resolved_module in dct[RESOLVED_MODULE_ENTRY_NAME]:
                        if isinstance(resolved_module, str):
                            resolved_classes.append(object_hook(json.loads(resolved_module)))
                    dct_obj[RESOLVED_MODULE_ENTRY_NAME] = resolved_classes
                if isinstance(key, str) and 'tf_source_modules' in key and 'file_path' in key:
                    tf_definition_key = json.loads(key)
                    tf_definition_key_obj = TFDefinitionKey(file_path=tf_definition_key["file_path"], tf_source_modules=object_hook(
                        tf_definition_key["tf_source_modules"]))
                    dct_obj[tf_definition_key_obj] = value
                    del dct_obj[key]
            return dct_obj
        return dct
    except (KeyError, TypeError, JSONDecodeError):
        return dct


def get_jsonpath_from_evaluated_key(evaluated_key: str) -> JSONPath:
    evaluated_key = evaluated_key.replace("/", ".")
    return parse(f"$..{evaluated_key}")  # type:ignore[no-any-return]



from typing import List, Any, Dict, Optional

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver
from checkov.common.checks_infra.solvers.complex_solvers.base_complex_solver import BaseComplexSolver
from functools import reduce
from operator import and_


class AndSolver(BaseComplexSolver):
    operator = Operators.AND  # noqa: CCE003  # a static attribute

    def __init__(self, solvers: List[BaseSolver], resource_types: List[str]) -> None:
        super().__init__(solvers, resource_types)

    def _get_operation(self, *args: Any, **kwargs: Any) -> Any:
        return reduce(and_, args)

    def get_operation(self, vertex: Dict[str, Any]) -> Optional[bool]:
        has_unrendered_attribute = False
        for solver in self.solvers:
            operation = solver.get_operation(vertex)
            if operation is None:
                has_unrendered_attribute = True
            elif not operation:
                return False
        return None if has_unrendered_attribute else True



from __future__ import annotations

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list, extract_policy_dict
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import Any


class IAMStarResourcePolicyDocument(BaseResourceCheck):
    def __init__(self) -> None:
        name = 'Ensure no IAM policies documents allow "*" as a statement\'s resource'
        id = "CKV_AWS_355"
        supported_resources = (
            "aws_iam_role_policy",
            "aws_iam_user_policy",
            "aws_iam_group_policy",
            "aws_iam_policy",
            "aws_ssoadmin_permission_set_inline_policy",
        )
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        policy_block = None

        policy = conf.get("policy")
        inline_policy = conf.get("inline_policy")
        if policy and isinstance(policy, list):
            policy_block = extract_policy_dict(policy[0])
        elif inline_policy and isinstance(inline_policy, list):
            policy_block = extract_policy_dict(inline_policy[0])

        if policy_block and "Statement" in policy_block:
            for statement in force_list(policy_block["Statement"]):
                if "Resource" in statement:
                    resources = force_list(statement["Resource"])
                    if (
                        isinstance(resources[0], str)
                        and statement.get("Effect", "Allow") == "Allow"
                        and "*" in resources
                    ):
                        # scanning a HCL file
                        return CheckResult.FAILED
                    elif (
                        isinstance(resources[0], list)
                        and statement.get("Effect", ["Allow"]) == ["Allow"]
                        and "*" in resources[0]
                    ):
                        # scanning a TF plan file
                        return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ["policy", "inline_policy"]


check = IAMStarResourcePolicyDocument()



from __future__ import annotations

import itertools
import logging
import os
import sys
from datetime import datetime
from pathlib import Path
from typing import TYPE_CHECKING, cast, Any
from checkov.common.output.common import format_string_to_licenses, validate_lines

from cyclonedx.model import (
    XsUri,
    ExternalReference,
    ExternalReferenceType,
    sha1sum,
    HashAlgorithm,
    HashType,
    LicenseChoice,
    License, Property,
)
from cyclonedx.model.bom import Bom, Tool
from cyclonedx.model.component import Component, ComponentType
from cyclonedx.model.vulnerability import (
    Vulnerability,
    VulnerabilityAdvisory,
    BomTarget,
    VulnerabilitySource,
    VulnerabilityRating,
    VulnerabilityScoreSource,
    VulnerabilitySeverity,
)
from cyclonedx.output import get_instance, OutputFormat
from packageurl import PackageURL

from checkov.common.output.common import ImageDetails
from checkov.common.output.report import CheckType
from checkov.common.output.cyclonedx_consts import (
    SCA_CHECKTYPES,
    PURL_TYPE_MAVEN,
    DEFAULT_CYCLONE_SCHEMA_VERSION,
    CYCLONE_SCHEMA_VERSION,
    FILE_NAME_TO_PURL_TYPE,
    IMAGE_DISTRO_TO_PURL_TYPE,
    TWISTCLI_PACKAGE_TYPE_TO_PURL_TYPE,
    BC_SEVERITY_TO_CYCLONEDX_LEVEL,
)
from checkov.common.output.record import SCA_PACKAGE_SCAN_CHECK_NAME
from checkov.common.sca.commons import UNFIXABLE_VERSION, get_fix_version
from checkov.common.util.consts import CHECKOV_DISPLAY_REGISTRY_URL

if sys.version_info >= (3, 8):
    from importlib.metadata import version as meta_version
else:
    from importlib_metadata import version as meta_version

if TYPE_CHECKING:
    from checkov.common.output.extra_resource import ExtraResource
    from checkov.common.output.record import Record
    from checkov.common.output.report import Report


class CycloneDX:
    def __init__(self, reports: list[Report], repo_id: str | None, export_iac_only: bool = False) -> None:
        self.repo_id = f"{repo_id}/" if repo_id is not None else ""
        self.reports = reports
        self.export_iac_only = export_iac_only

        self.bom = self.create_bom()

    def create_bom(self) -> Bom:
        bom = Bom()

        try:
            version = meta_version("checkov")  # type:ignore[no-untyped-call]
        except Exception:
            # Unable to determine current version of 'checkov'
            version = "UNKNOWN"

        this_tool = Tool(vendor="bridgecrew", name="checkov", version=version)
        self.update_tool_external_references(this_tool)
        bom.metadata.tools.add(this_tool)

        for report in self.reports:
            if report.check_type in SCA_CHECKTYPES and self.export_iac_only:
                continue

            # if the report is of SCA_IMAGE type, we should add to the report one image component per image
            is_image_report = report.check_type == CheckType.SCA_IMAGE
            image_resources_for_image_components = {}

            for check in itertools.chain(report.passed_checks, report.skipped_checks):
                if report.check_type in SCA_CHECKTYPES and check.check_name != SCA_PACKAGE_SCAN_CHECK_NAME:
                    continue
                component = self.create_component(check_type=report.check_type, resource=check)

                if not bom.has_component(component=component):
                    bom.components.add(component)

                if is_image_report and check.file_path not in image_resources_for_image_components:
                    image_resources_for_image_components[check.file_path] = check

            for check in report.failed_checks:
                if report.check_type in SCA_CHECKTYPES and check.check_name != SCA_PACKAGE_SCAN_CHECK_NAME:
                    continue
                component = self.create_component(check_type=report.check_type, resource=check)

                if bom.has_component(component=component):
                    component = (
                        bom.get_component_by_purl(  # type:ignore[assignment]  # the previous line checks, if exists
                            purl=component.purl
                        )
                    )

                vulnerability = self.create_vulnerability(
                    check_type=report.check_type, resource=check, component=component
                )

                component.add_vulnerability(vulnerability)
                bom.components.add(component)

                if is_image_report:
                    if check.file_path not in image_resources_for_image_components:
                        image_resources_for_image_components[check.file_path] = check

            for resource in sorted(report.extra_resources):
                component = self.create_component(check_type=report.check_type, resource=resource)

                if not bom.has_component(component=component):
                    bom.components.add(component)

            if is_image_report:
                for image_resource in image_resources_for_image_components:
                    self.create_image_component(resource=image_resources_for_image_components[image_resource], bom=bom)

        return bom

    def create_component(self, check_type: str, resource: Record | ExtraResource) -> Component:
        """Creates a component"""
        # purl structure conventions: https://github.com/package-url/purl-spec/blob/master/PURL-TYPES.rst

        if check_type in SCA_CHECKTYPES:
            component = self.create_library_component(check_type=check_type, resource=resource)
        else:
            component = self.create_application_component(check_type=check_type, resource=resource)

        return component

    def create_application_component(self, check_type: str, resource: Record | ExtraResource) -> Component:
        """Creates an application component
        Ex.
        <component bom-ref="pkg:terraform/cli_repo/pd/main.tf/aws_s3_bucket.example@sha1:c9b9b2eba0a7d4ccb66096df77e1a6715ea1ae85" type="application">
          <name>aws_s3_bucket.example</name>
          <version>sha1:c9b9b2eba0a7d4ccb66096df77e1a6715ea1ae85</version>
          <hashes>
            <hash alg="SHA-1">c9b9b2eba0a7d4ccb66096df77e1a6715ea1ae85</hash>
          </hashes>
          <purl>pkg:terraform/cli_repo/pd/main.tf/aws_s3_bucket.example@sha1:c9b9b2eba0a7d4ccb66096df77e1a6715ea1ae85</purl>
        </component>
        """

        sha1_hash = sha1sum(filename=resource.file_abs_path)
        purl = PackageURL(
            type=check_type,
            namespace=f"{self.repo_id}/{resource.file_path}",
            name=resource.resource,
            version=f"sha1:{sha1_hash}",
        )
        component = Component(
            bom_ref=str(purl),
            name=resource.resource,
            version=f"sha1:{sha1_hash}",
            hashes=[
                HashType(
                    algorithm=HashAlgorithm.SHA_1,
                    hash_value=sha1_hash,
                )
            ],
            component_type=ComponentType.APPLICATION,
            purl=purl,
        )
        return component

    def create_library_component(self, resource: Record | ExtraResource, check_type: str) -> Component:
        """Creates a library component
        Ex.
        <component bom-ref="pkg:pypi/cli_repo/pd/requirements.txt/flask@0.6" type="library">
          <name>flask</name>
          <version>0.6</version>
          <purl>pkg:pypi/cli_repo/pd/requirements.txt/flask@0.6</purl>
          <properties>
            <property name="startLine">5</property>
            <property name="endLine">6</property>
          </properties>
        </component>
        """

        if not resource.vulnerability_details:
            # this shouldn't happen
            logging.error(f"Resource {resource.resource} doesn't have 'vulnerability_details' set")
            return Component(name="unknown")
        qualifiers = None
        file_name = Path(resource.file_path).name
        if check_type is CheckType.SCA_IMAGE:
            package_type = resource.vulnerability_details['package_type']
            image_distro_name = resource.vulnerability_details.get('image_details', ImageDetails()).distro.split(' ')[0]
            file_path = resource.file_path.split(' ')[0]
            if package_type == 'os':
                purl_type = IMAGE_DISTRO_TO_PURL_TYPE.get(image_distro_name, 'generic')
                namespace = f'{self.repo_id}/{file_path}/{image_distro_name.lower()}'
                qualifiers = f'distro={resource.vulnerability_details.get("image_details", ImageDetails()).distro_release}'
            else:
                purl_type = TWISTCLI_PACKAGE_TYPE_TO_PURL_TYPE.get(package_type, 'generic')
                namespace = f"{self.repo_id}/{file_path}"
        else:
            purl_type = FILE_NAME_TO_PURL_TYPE.get(file_name, "generic")
            namespace = f"{self.repo_id}/{resource.file_path}"
            registry_url = resource.vulnerability_details.get("package_registry")
            is_private_registry = resource.vulnerability_details.get("is_private_registry", False)
            if CHECKOV_DISPLAY_REGISTRY_URL and registry_url and is_private_registry:
                qualifiers = f'registry_url={registry_url}'
        package_group = None
        package_name = resource.vulnerability_details["package_name"]
        package_version = resource.vulnerability_details["package_version"]

        if purl_type == PURL_TYPE_MAVEN and "_" in package_name:
            package_group, package_name = package_name.split("_", maxsplit=1)
            namespace += f"/{package_group}"

            if not package_name:
                logging.info('maven package name format not as expected')
                package_name = resource.vulnerability_details["package_name"]

        # add licenses, if exists
        license_choices = None
        licenses = resource.vulnerability_details.get("licenses")

        if licenses:
            license_choices = [
                LicenseChoice(license_=License(license_name=license)) for license in format_string_to_licenses(licenses)
            ]

        purl = PackageURL(
            type=purl_type,
            namespace=namespace,
            name=package_name,
            version=package_version,
            qualifiers=qualifiers,
        )

        lines = resource.file_line_range
        lines = validate_lines(lines)
        properties = None
        if lines:
            properties = [Property(name="endLine", value=str(lines[1])), Property(name="startLine", value=str(lines[0]))]

        component = Component(
            bom_ref=str(purl),
            group=package_group,
            name=package_name,
            version=package_version,
            component_type=ComponentType.LIBRARY,
            licenses=license_choices,
            purl=purl,
            properties=properties
        )

        return component

    def create_image_component(self, resource: Record, bom: Bom) -> None:
        image_id = cast("dict[str, Any]", resource.vulnerability_details).get('image_details', ImageDetails()).image_id
        file_path = resource.file_path.split(' ')[0]
        image_purl = PackageURL(
            type='oci',
            namespace=self.repo_id,
            name=file_path,
            version=image_id,
        )
        bom.components.add(
            Component(
                bom_ref=str(image_purl),
                component_type=ComponentType.CONTAINER,
                name=f"{self.repo_id}/{image_id}",
                version="",
                purl=image_purl,
            )
        )

    def create_vulnerability(self, check_type: str, resource: Record, component: Component) -> Vulnerability:
        """Creates a vulnerability"""

        if check_type in SCA_CHECKTYPES:
            vulnerability = self.create_cve_vulnerability(resource=resource, component=component)
        else:
            vulnerability = self.create_iac_vulnerability(resource=resource, component=component)

        return vulnerability

    def create_iac_vulnerability(self, resource: Record, component: Component) -> Vulnerability:
        """Creates a IaC based vulnerability
        Ex.
        <vulnerability bom-ref="41f657e7-a83b-4535-9b83-541211d02397">
          <id>CKV_AWS_21</id>
          <source>
            <name>checkov</name>
          </source>
          <ratings>
            <rating>
              <severity>medium</severity>
            </rating>
          </ratings>
          <description>Resource: aws_s3_bucket.example. Ensure all data stored in the S3 bucket have versioning enabled</description>
          <advisories>
            <advisory>
              <url>https://docs.bridgecrew.io/docs/s3_16-enable-versioning</url>
            </advisory>
          </advisories>
          <affects>
            <target>
              <ref>pkg:terraform/main.tf/aws_s3_bucket.example@sha1:c9b9b2eba0a7d4ccb66096df77e1a6715ea1ae85</ref>
            </target>
          </affects>
        </vulnerability>
        """

        advisories = None
        if resource.guideline:
            advisories = [VulnerabilityAdvisory(url=XsUri(resource.guideline))]

        severity = VulnerabilitySeverity.UNKNOWN
        if resource.severity:
            severity = BC_SEVERITY_TO_CYCLONEDX_LEVEL.get(resource.severity.name, VulnerabilitySeverity.UNKNOWN)

        vulnerability = Vulnerability(
            id=resource.check_id,
            source=VulnerabilitySource(name="checkov"),
            ratings=[
                VulnerabilityRating(
                    severity=severity,
                )
            ],
            description=f"Resource: {resource.resource}. {resource.check_name}",
            affects_targets=[BomTarget(ref=component.bom_ref.value)],
            advisories=advisories,
        )
        return vulnerability

    def create_cve_vulnerability(self, resource: Record, component: Component) -> Vulnerability:
        """Creates a CVE based vulnerability
        Ex.
        <vulnerability bom-ref="f18f3674-092f-4e9a-8452-641fd11fc70f">
          <id>CVE-2019-1010083</id>
          <source>
            <url>https://nvd.nist.gov/vuln/detail/CVE-2019-1010083</url>
          </source>
          <ratings>
            <rating>
              <source>
                <url>https://nvd.nist.gov/vuln/detail/CVE-2019-1010083</url>
              </source>
              <score>7.5</score>
              <severity>unknown</severity>
              <method>CVSSv3</method>
              <vector>AV:N/AC:L/PR:N/UI:N/S:U/C:N/I:N/A:H</vector>
            </rating>
          </ratings>
          <description>The Pallets Project Flask before 1.0 is affected by: unexpected memory usage. ...</description>
          <recommendation>fixed in 1.0</recommendation>
          <published>2019-07-17T14:15:00</published>
          <affects>
            <target>
              <ref>pkg:pypi/flask@0.6</ref>
            </target>
          </affects>
        </vulnerability>
        """

        if not resource.vulnerability_details:
            # this shouldn't happen
            logging.error(f"Resource {resource.resource} doesn't have 'vulnerability_details' set")
            return Vulnerability()

        severity = VulnerabilitySeverity.UNKNOWN
        if resource.severity:
            severity = BC_SEVERITY_TO_CYCLONEDX_LEVEL.get(resource.severity.name, VulnerabilitySeverity.UNKNOWN)

        source = None
        source_url = resource.vulnerability_details.get("link")
        if source_url:
            source = VulnerabilitySource(url=source_url)
        method = None
        vector = resource.vulnerability_details["vector"]

        if vector:
            method = VulnerabilityScoreSource.get_from_vector(vector)
            vector = method.get_localised_vector(vector)

        fix_version = self.get_fix_version_overview(resource.vulnerability_details)
        vulnerability = Vulnerability(
            id=resource.vulnerability_details["id"],
            source=source,
            ratings=[
                VulnerabilityRating(
                    source=source,
                    score=resource.vulnerability_details.get("cvss"),
                    severity=severity,
                    method=method,
                    vector=vector,
                )
            ],
            description=resource.vulnerability_details.get("description"),
            recommendation=fix_version,
            published=datetime.fromisoformat(resource.vulnerability_details["published_date"].replace("Z", "")),
            affects_targets=[BomTarget(ref=component.bom_ref.value)],
        )
        return vulnerability

    def get_fix_version_overview(self, vulnerability_details: dict[str, Any]) -> str | None:
        is_private_fix = vulnerability_details.get("is_private_fix")
        public_fix_version_prefix = "No private fix available. " if is_private_fix is False else ""
        fix_version: str = get_fix_version(vulnerability_details)
        return f'{public_fix_version_prefix}Fixed in {fix_version}' if fix_version and fix_version != UNFIXABLE_VERSION else fix_version

    def get_output(self, output_format: OutputFormat) -> str:
        """Returns the SBOM as a formatted string"""

        schema_version = CYCLONE_SCHEMA_VERSION.get(
            os.getenv("CHECKOV_CYCLONEDX_SCHEMA_VERSION", ""), DEFAULT_CYCLONE_SCHEMA_VERSION
        )
        output = get_instance(
            bom=self.bom,
            output_format=output_format,
            schema_version=schema_version,
        ).output_as_string()

        return output

    def get_xml_output(self) -> str:
        """Returns the SBOM as a XML formatted string"""

        return self.get_output(output_format=OutputFormat.XML)

    def get_json_output(self) -> str:
        """Returns the SBOM as a JSON formatted string"""

        return self.get_output(output_format=OutputFormat.JSON)

    def update_tool_external_references(self, tool: Tool) -> None:
        tool.external_references.update(
            [
                ExternalReference(
                    reference_type=ExternalReferenceType.BUILD_SYSTEM,
                    url=XsUri("https://github.com/bridgecrewio/checkov/actions"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.DISTRIBUTION,
                    url=XsUri("https://pypi.org/project/checkov/"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.DOCUMENTATION,
                    url=XsUri("https://www.checkov.io/1.Welcome/What%20is%20Checkov.html"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.ISSUE_TRACKER,
                    url=XsUri("https://github.com/bridgecrewio/checkov/issues"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.LICENSE,
                    url=XsUri("https://github.com/bridgecrewio/checkov/blob/master/LICENSE"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.SOCIAL,
                    url=XsUri("https://twitter.com/bridgecrewio"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.VCS,
                    url=XsUri("https://github.com/bridgecrewio/checkov"),
                ),
                ExternalReference(
                    reference_type=ExternalReferenceType.WEBSITE,
                    url=XsUri("https://www.checkov.io/"),
                ),
            ]
        )



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class LogAuditRDSEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure log audit is enabled for RDS"
        id = "CKV_ALI_38"
        supported_resources = ("alicloud_log_audit",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "variable_map/[0]/rds_enabled"


check = LogAuditRDSEnabled()



from json import JSONDecodeError

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
import json
from typing import List


class S3AllowsAnyPrincipal(BaseResourceCheck):

    def __init__(self):
        name = "Ensure S3 bucket does not allow an action with any Principal"
        id = "CKV_AWS_70"
        supported_resources = ['aws_s3_bucket', 'aws_s3_bucket_policy']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'policy' not in conf.keys():
            return CheckResult.UNKNOWN
        if not isinstance(conf['policy'][0], str):
            policy_block = conf['policy'][0]
        else:
            if "data.aws_iam_policy_document" in conf['policy'][0]:
                return CheckResult.UNKNOWN
            else:
                try:
                    policy_block = json.loads(conf['policy'][0])
                except JSONDecodeError:  # nosec
                    return CheckResult.UNKNOWN

        if isinstance(policy_block, dict) and 'Statement' in policy_block.keys():
            for statement in force_list(policy_block['Statement']):
                if statement['Effect'] == 'Deny' or 'Principal' not in statement:
                    continue
                principal = statement['Principal']
                if principal == '*':
                    return CheckResult.FAILED
                if 'AWS' in statement['Principal']:
                    # Can be a string or an array of strings
                    aws = statement['Principal']['AWS']
                    if (isinstance(aws, str) and aws == '*') or (isinstance(aws, list) and '*' in aws):
                        return CheckResult.FAILED

        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['policy']


check = S3AllowsAnyPrincipal()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class DMSS3DefinesIntransitEncryption(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure DMS S3 defines in-transit encryption"
        id = "CKV_AWS_299"
        supported_resources = ("aws_dms_s3_endpoint",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'ssl_mode'

    def get_expected_values(self) -> list[Any]:
        return ["require", "verify-ca", "verify-full"]


check = DMSS3DefinesIntransitEncryption()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class WorkspaceUserVolumeEncrypted(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that Workspace user volumes are encrypted"
        id = "CKV_AWS_155"
        supported_resources = ['aws_workspaces_workspace']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'user_volume_encryption_enabled'

    def get_expected_value(self):
        return True


check = WorkspaceUserVolumeEncrypted()



from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class S3IgnorePublicACLs(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure S3 bucket has ignore public ACLs enabled"
        id = "CKV_AWS_55"
        supported_resources = ("AWS::S3::Bucket",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/PublicAccessBlockConfiguration/IgnorePublicAcls"


check = S3IgnorePublicACLs()



from __future__ import annotations

import os
from typing import Any

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.images.graph.image_referencer_provider import GraphImageReferencerProvider
from checkov.common.images.image_referencer import Image
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.common.util.str_utils import removeprefix


class BaseKubernetesProvider(GraphImageReferencerProvider):

    def extract_images_from_resources(self) -> list[Image]:
        images = []

        supported_resources_graph = self.extract_nodes()

        for resource in self.extract_resource(supported_resources_graph):
            resource_type = resource[CustomAttributes.RESOURCE_TYPE]
            resource_path = self._get_resource_path(resource)

            extract_images_func = self.supported_resource_types.get(resource_type)
            if extract_images_func:
                for name in extract_images_func(resource):
                    images.append(
                        Image(
                            file_path=resource_path,
                            name=name,
                            start_line=resource[START_LINE],
                            end_line=resource[END_LINE],
                            related_resource_id=f'{removeprefix(resource_path, os.getenv("BC_ROOT_DIR", ""))}:{resource.get("id_")}',
                        )
                    )

        return images

    def _get_resource_path(self, resource: dict[str, Any]) -> str:
        return resource.get(CustomAttributes.FILE_PATH, "")



from __future__ import annotations

from typing import Any

from checkov.common.util.type_forcers import force_int
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class BranchProtectionReviewNumTwo(BaseResourceCheck):
    def __init__(self) -> None:
        name = "GitHub pull requests should require at least 2 approvals"
        id = "CKV_GIT_5"
        supported_resources = ("github_branch_protection_v3", "github_branch_protection")
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        reviews = conf.get("required_pull_request_reviews")
        if reviews and isinstance(reviews, list):
            review_count = reviews[0].get("required_approving_review_count")
            if review_count and isinstance(review_count, list):
                count = force_int(review_count[0])
                if count and count >= 2:
                    return CheckResult.PASSED
        self.evaluated_keys = ["required_pull_request_reviews/[0]/required_approving_review_count"]
        return CheckResult.FAILED


check = BranchProtectionReviewNumTwo()



from checkov.common.vcs.vcs_schema import VCSSchema


class RepositoryCollaboratorsSchema(VCSSchema):
    def __init__(self) -> None:
        schema = {
            "type": "array",
            "items": {
                "title": "Collaborator",
                "description": "Collaborator",
                "type": "object",
                "properties": {
                    "login": {
                        "type": "string",
                        "examples": [
                            "octocat"
                        ]
                    },
                    "id": {
                        "type": "integer",
                        "examples": [
                            1
                        ]
                    },
                    "email": {
                        "type": [
                            "string",
                            "null"
                        ]
                    },
                    "name": {
                        "type": [
                            "string",
                            "null"
                        ]
                    },
                    "node_id": {
                        "type": "string",
                        "examples": [
                            "MDQ6VXNlcjE="
                        ]
                    },
                    "avatar_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://github.com/images/error/octocat_happy.gif"
                        ]
                    },
                    "gravatar_id": {
                        "type": [
                            "string",
                            "null"
                        ],
                        "examples": [
                            "41d064eb2195891e12d0413f63227ea7"
                        ]
                    },
                    "url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/users/octocat"
                        ]
                    },
                    "html_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://github.com/octocat"
                        ]
                    },
                    "followers_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/users/octocat/followers"
                        ]
                    },
                    "following_url": {
                        "type": "string",
                        "examples": [
                            "https://api.github.com/users/octocat/following{/other_user}"
                        ]
                    },
                    "gists_url": {
                        "type": "string",
                        "examples": [
                            "https://api.github.com/users/octocat/gists{/gist_id}"
                        ]
                    },
                    "starred_url": {
                        "type": "string",
                        "examples": [
                            "https://api.github.com/users/octocat/starred{/owner}{/repo}"
                        ]
                    },
                    "subscriptions_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/users/octocat/subscriptions"
                        ]
                    },
                    "organizations_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/users/octocat/orgs"
                        ]
                    },
                    "repos_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/users/octocat/repos"
                        ]
                    },
                    "events_url": {
                        "type": "string",
                        "examples": [
                            "https://api.github.com/users/octocat/events{/privacy}"
                        ]
                    },
                    "received_events_url": {
                        "type": "string",
                        "format": "uri",
                        "examples": [
                            "https://api.github.com/users/octocat/received_events"
                        ]
                    },
                    "type": {
                        "type": "string",
                        "examples": [
                            "User"
                        ]
                    },
                    "site_admin": {
                        "type": "boolean"
                    },
                    "permissions": {
                        "type": "object",
                        "properties": {
                            "pull": {
                                "type": "boolean"
                            },
                            "triage": {
                                "type": "boolean"
                            },
                            "push": {
                                "type": "boolean"
                            },
                            "maintain": {
                                "type": "boolean"
                            },
                            "admin": {
                                "type": "boolean"
                            }
                        },
                        "required": [
                            "pull",
                            "push",
                            "admin"
                        ]
                    },
                    "role_name": {
                        "type": "string",
                        "examples": [
                            "admin"
                        ]
                    }
                },
                "required": [
                    "avatar_url",
                    "events_url",
                    "followers_url",
                    "following_url",
                    "gists_url",
                    "gravatar_id",
                    "html_url",
                    "id",
                    "node_id",
                    "login",
                    "organizations_url",
                    "received_events_url",
                    "repos_url",
                    "site_admin",
                    "starred_url",
                    "subscriptions_url",
                    "type",
                    "url",
                    "role_name"
                ]
            }
        }
        super().__init__(schema=schema)


schema = RepositoryCollaboratorsSchema()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult
from checkov.github.base_github_org_check import BaseOrganizationCheck


class GithubRequireOrganizationIsVerified(BaseOrganizationCheck):
    def __init__(self) -> None:
        name = "Ensure an organization's identity is confirmed with a Verified badge Passed"
        id = "CKV_GITHUB_28"
        super().__init__(id=id, name=name, missing_attribute_result=CheckResult.FAILED)

    def get_evaluated_keys(self) -> list[str]:
        return ["is_verified"]

    def get_allowed_values(self) -> list[Any]:
        return [True]


check = GithubRequireOrganizationIsVerified()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class AbsSecurityGroupUnrestrictedIngress(BaseResourceCheck):
    def __init__(self, check_id, port):
        name = "Ensure no security groups allow ingress from 0.0.0.0:0 to port %d" % port
        supported_resources = ['AWS::EC2::SecurityGroup', 'AWS::EC2::SecurityGroupIngress']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def scan_resource_conf(self, conf):
        """
        Looks for configuration at security group ingress rules:
        https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html
        :param conf: aws_security_group configuration
        :return: <CheckResult>
        """
        rules = []
        if conf['Type'] == 'AWS::EC2::SecurityGroup':
            if 'Properties' in conf.keys():
                if 'SecurityGroupIngress' in conf['Properties'].keys():
                    rules = conf['Properties']['SecurityGroupIngress']
        elif conf['Type'] == 'AWS::EC2::SecurityGroupIngress':
            if 'Properties' in conf.keys():
                rules = []
                rules.append(conf['Properties'])

        if not isinstance(rules, list):
            return CheckResult.UNKNOWN

        for rule in rules:
            if rule.__contains__('FromPort') and rule.__contains__('ToPort'):
                if (isinstance(rule['FromPort'], int) and isinstance(rule['ToPort'], int)) or \
                        isinstance(rule['FromPort'], str) and isinstance(rule['ToPort'], str):
                    if self.range(rule):
                        if 'CidrIp' in rule.keys():
                            cidr = rule['CidrIp']
                            if cidr == '0.0.0.0/0':  # nosec  # nosec
                                return CheckResult.FAILED
                        elif 'CidrIpv6' in rule.keys() and \
                                rule['CidrIpv6'] in ['::/0', '0000:0000:0000:0000:0000:0000:0000:0000/0']:
                            return CheckResult.FAILED
        return CheckResult.PASSED

    def range(self, rule):
        if int(rule['FromPort']) <= int(self.port) <= int(rule['ToPort']):
            return True
        return False



from __future__ import annotations

from checkov.kubernetes.graph_builder.graph_components.edge_builders.K8SEdgeBuilder import K8SEdgeBuilder
from checkov.kubernetes.graph_builder.graph_components.blocks import KubernetesBlock
from checkov.kubernetes.graph_builder.graph_components.ResourceKeywordIdentifier import ResourceKeywordIdentifier
from checkov.kubernetes.kubernetes_utils import FILTERED_RESOURCES_FOR_EDGE_BUILDERS


class KeywordEdgeBuilder(K8SEdgeBuilder):

    @staticmethod
    def should_search_for_edges(vertex: KubernetesBlock) -> bool:
        return vertex.attributes.get("kind") in ResourceKeywordIdentifier.KINDS_KEYWORDS_MAP.keys() \
            and vertex.attributes.get("kind") not in FILTERED_RESOURCES_FOR_EDGE_BUILDERS

    @staticmethod
    def find_connections(vertex: KubernetesBlock, vertices: list[KubernetesBlock]) -> list[int]:
        """
        connection is defined by a match between a vertex's (of a certain type) references definitions to a potential
        vertex (of any type).

        example:
        A Pod with the property 'spec.serviceAccountName' with value 'service-123' will match a resource of type
        'ServiceAccount' with a 'metadata.name' property equals to 'service-123'
        """

        connections: list[int] = []
        for potential_vertex_index, potential_vertex in enumerate(vertices):
            if potential_vertex.id == vertex.id:
                continue
            resource_references_definitions: list[dict[str, str] | list[dict[str, dict[str, str]]]] = ResourceKeywordIdentifier.KINDS_KEYWORDS_MAP[vertex.attributes["kind"]]  # type: ignore[assignment]
            # check that resource items comply to all references definitions defined in ResourceKeywordIdentifier
            for references_definition in resource_references_definitions:
                match = True

                if isinstance(references_definition, dict):
                    for potential_vertex_key, vertex_key in references_definition.items():
                        match = KeywordEdgeBuilder._find_match_in_attributes(vertex, potential_vertex, potential_vertex_key, vertex_key, match)
                    if match:
                        connections.append(potential_vertex_index)

                # some items are nested in lists and their value in the vertex is concatenated with their index,
                # like so:  subjects.0.name
                elif isinstance(references_definition, list):
                    # not really a loop, just extracting the dict's key
                    for base_key_attribute, reference_definitions_items in references_definition[0].items():
                        vertex_attribute_references_list: list[dict[str, str]] = vertex.attributes.get(base_key_attribute)  # type: ignore[assignment]
                        if not vertex_attribute_references_list:
                            continue
                        # iterate every item on the list as a separate resource
                        for i in range(len(vertex_attribute_references_list)):
                            match = True
                            for potential_vertex_key, vertex_key in reference_definitions_items.items():
                                vertex_key = f"{base_key_attribute}.{i}.{vertex_key}"
                                match = KeywordEdgeBuilder._find_match_in_attributes(vertex, potential_vertex, potential_vertex_key, vertex_key, match)
                            if match:
                                connections.append(potential_vertex_index)

        return connections

    @staticmethod
    def _find_match_in_attributes(vertex: KubernetesBlock,
                                  potential_vertex: KubernetesBlock,
                                  potential_vertex_key: str,
                                  vertex_key: str,
                                  match: bool) -> bool:

        vertex_ref = vertex.attributes.get(vertex_key)
        potential_vertex_ref = potential_vertex.attributes.get(potential_vertex_key)
        if vertex_ref is None or potential_vertex_ref is None or vertex_ref != potential_vertex_ref:
            # if not all attributes match then it's not qualified as an edge
            match = False

        return match



from __future__ import annotations

import re
from typing import Generator, Set, Tuple
from typing import Any
from typing import TYPE_CHECKING

from detect_secrets.plugins.high_entropy_strings import Base64HighEntropyString
from detect_secrets.plugins.high_entropy_strings import HexHighEntropyString
from detect_secrets.plugins.keyword import KeywordDetector
from detect_secrets.plugins.keyword import DENYLIST
from detect_secrets.plugins.keyword import AFFIX_REGEX
from detect_secrets.plugins.keyword import CLOSING
from detect_secrets.plugins.keyword import OPTIONAL_WHITESPACE
from detect_secrets.plugins.keyword import QUOTE
from detect_secrets.plugins.keyword import SECRET
from detect_secrets.plugins.base import BasePlugin

from detect_secrets.util.filetype import determine_file_type
from checkov.secrets.plugins.detector_utils import SINGLE_LINE_PARSER, MULTILINE_PARSERS, \
    REGEX_VALUE_KEYWORD_BY_FILETYPE, REGEX_VALUE_SECRET_BY_FILETYPE, remove_fp_secrets_in_keys, detect_secret, \
    analyze_multiline_keyword_combinator, mark_set_multiline

from checkov.secrets.runner import SOURCE_CODE_EXTENSION

if TYPE_CHECKING:
    from detect_secrets.core.potential_secret import PotentialSecret
    from detect_secrets.util.code_snippet import CodeSnippet

MAX_LINE_LENGTH = 10000
MAX_KEYWORD_LIMIT = 500
ENTROPY_KEYWORD_COMBINATOR_LIMIT = 3
ENTROPY_KEYWORD_LIMIT = 4.5

DENY_LIST_REGEX = r'|'.join(DENYLIST)
# Support for suffix after keyword i.e. password_secure = "value"
DENY_LIST_REGEX2 = r'({denylist}){suffix}'.format(
    denylist=DENY_LIST_REGEX,
    suffix=AFFIX_REGEX,
)

KEY = r'{words}({closing})?'.format(
    words=AFFIX_REGEX,
    closing=CLOSING,
)

FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX = re.compile(
    # e.g. var: MY_PASSWORD_123
    r'{whitespace}({key})?:{whitespace}({quote}?){words}{denylist}({closing})?(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        words=AFFIX_REGEX,
        denylist=DENY_LIST_REGEX2,
        closing=CLOSING,
    ),
    flags=re.IGNORECASE,
)

QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX = re.compile(
    # e.g. var: MY_PASSWORD_123
    r'{whitespace}"({key})?":{whitespace}("?){words}{denylist}({closing})?(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        words=AFFIX_REGEX,
        denylist=DENY_LIST_REGEX2,
        closing=CLOSING,
    ),
    flags=re.IGNORECASE,
)

FOLLOWED_BY_COLON_VALUE_SECRET_REGEX = re.compile(
    # e.g. var: Zmlyc3Rfc2VjcmV0X2hlcmVfd2hvYV9tdWx0aWxsaW5lX3Nob3VsZF93b3JrXzE==
    r'{whitespace}({key})?:{whitespace}({quote}?)({secret})(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        secret=SECRET,
    ),
    flags=re.IGNORECASE,
)

QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_SECRET_REGEX = re.compile(
    # e.g. var: Zmlyc3Rfc2VjcmV0X2hlcmVfd2hvYV9tdWx0aWxsaW5lX3Nob3VsZF93b3JrXzE==
    r'{whitespace}"({key})?":{whitespace}("?)({secret})(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        secret=SECRET,
    ),
    flags=re.IGNORECASE,
)

FOLLOWED_BY_EQUAL_VALUE_KEYWORD_REGEX = re.compile(
    # e.g. var = MY_PASSWORD_123
    r'{whitespace}({key})?={whitespace}({quote}?){words}{denylist}({closing})?(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        words=AFFIX_REGEX,
        denylist=DENY_LIST_REGEX2,
        closing=CLOSING,
    ),
    flags=re.IGNORECASE,
)

FOLLOWED_BY_EQUAL_VALUE_SECRET_REGEX = re.compile(
    # e.g. var = Zmlyc3Rfc2VjcmV0X2hlcmVfd2hvYV9tdWx0aWxsaW5lX3Nob3VsZF93b3JrXzE==
    r'{whitespace}({key})?={whitespace}({quote}?)({secret})(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        secret=SECRET,
    ),
    flags=re.IGNORECASE,
)


class EntropyKeywordCombinator(BasePlugin):
    secret_type = ""  # nosec  # noqa: CCE003  # a static attribute

    def __init__(self, limit: float = ENTROPY_KEYWORD_LIMIT, max_line_length: int = MAX_LINE_LENGTH) -> None:
        iac_limit = ENTROPY_KEYWORD_COMBINATOR_LIMIT
        self.high_entropy_scanners_iac = (Base64HighEntropyString(limit=iac_limit), HexHighEntropyString(limit=iac_limit))
        self.high_entropy_scanners = (Base64HighEntropyString(limit=limit), HexHighEntropyString(limit=limit))
        self.keyword_scanner = KeywordDetector()
        self.max_line_length = max_line_length

    def analyze_string(self, string: str) -> Generator[str, None, None]:
        yield ""

    def analyze_line(
            self,
            filename: str,
            line: str,
            line_number: int = 0,
            context: CodeSnippet | None = None,
            raw_context: CodeSnippet | None = None,
            **kwargs: Any,
    ) -> set[PotentialSecret]:
        if len(line) > self.max_line_length:
            # to keep good performance we skip long lines
            return set()

        is_iac = f".{filename.split('.')[-1]}" not in SOURCE_CODE_EXTENSION
        keyword_on_key = self.keyword_scanner.analyze_line(filename, line, line_number, **kwargs)
        if is_iac:
            filetype = determine_file_type(filename)
            single_line_parser = SINGLE_LINE_PARSER.get(filetype)
            multiline_parsers = MULTILINE_PARSERS.get(filetype)

            # classic key-value pair
            if keyword_on_key:
                if single_line_parser:
                    # Getting last detected one as only 1 violation available for line
                    secret_value, quoted_secret = EntropyKeywordCombinator.receive_last_secret_detected(keyword_on_key)
                    line = quoted_secret if quoted_secret else line
                    return single_line_parser.detect_secret(
                        scanners=self.high_entropy_scanners_iac,
                        filename=filename,
                        raw_context=raw_context,
                        line=line,
                        line_number=line_number,
                        kwargs=kwargs
                    )
                else:
                    # preprocess line before detecting secrets - add quotes on potential secrets to allow triggering
                    # entropy detector
                    secret_value, quoted_secret = EntropyKeywordCombinator.receive_last_secret_detected(keyword_on_key)
                    if line.find(quoted_secret) < 0:    # replace potential secret with quoted version
                        line = line.replace(secret_value, quoted_secret, 1)
                    detected_secrets = detect_secret(
                        scanners=self.high_entropy_scanners_iac,
                        filename=filename,
                        line=line,
                        line_number=line_number,
                        kwargs=kwargs
                    )
                    # postprocess detected secrets - filter out potential secrets on keyword
                    remove_fp_secrets_in_keys(detected_secrets, line)
                    return detected_secrets

            # not so classic key-value pair, from multiline, that is only in an array format.
            # The scan searches forwards and backwards for a potential secret pair, so no duplicates expected.
            elif multiline_parsers:
                # iterate over multiple parser and their related file type.
                # this is needed for file types, which embed other file type parser, ex Terraform with heredoc
                for parser_file_type, multiline_parser in multiline_parsers:
                    value_keyword_regex_to_group = REGEX_VALUE_KEYWORD_BY_FILETYPE.get(parser_file_type)
                    secret_keyword_regex_to_group = REGEX_VALUE_SECRET_BY_FILETYPE.get(parser_file_type)

                    potential_secrets = analyze_multiline_keyword_combinator(
                        filename=filename,
                        scanners=self.high_entropy_scanners,
                        multiline_parser=multiline_parser,
                        line_number=line_number,
                        context=context,
                        raw_context=raw_context,
                        value_pattern=value_keyword_regex_to_group,
                        secret_pattern=secret_keyword_regex_to_group,
                        kwargs=kwargs
                    )

                    if potential_secrets:
                        mark_set_multiline(potential_secrets)
                        # return a possible secret, otherwise check with next parser
                        return potential_secrets
        else:
            return detect_secret(
                # If we found a keyword (i.e. db_pass = ), lower the threshold to the iac threshold
                scanners=self.high_entropy_scanners if not keyword_on_key else self.high_entropy_scanners_iac,
                filename=filename,
                line=line,
                line_number=line_number,
                kwargs=kwargs
            )

        return set()

    @staticmethod
    def convert_to_reduce_noise(secret_value: str) -> str:
        # For empty string return empty string
        if not secret_value:
            return ''
        return f"\"{secret_value}\""

    @staticmethod
    def receive_last_secret_detected(keyword_on_key: Set[PotentialSecret]) -> Tuple[str, str]:
        return_value = ''
        quoted_return_value = ''
        for pt in keyword_on_key:
            if pt.secret_value:
                return_value = pt.secret_value
                quoted_return_value = EntropyKeywordCombinator.convert_to_reduce_noise(pt.secret_value)
        return return_value, quoted_return_value



from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list
import ast


class IAMStarActionPolicyDocument(BaseResourceCheck):
    def __init__(self):
        name = "Ensure no IAM policies documents allow \"*\" as a statement's actions"
        id = "CKV_AWS_63"
        supported_resources = ['AWS::IAM::Policy', 'AWS::IAM::Group', 'AWS::IAM::Role', 'AWS::IAM::User']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        myproperties = conf.get("Properties")
        type = conf['Type']

        # catch for inline policies
        if type != 'AWS::IAM::Policy':
            if isinstance(myproperties, dict) and 'Policies' in myproperties.keys():
                policies = myproperties['Policies']
                if len(policies) > 0:
                    for policy in policies:
                        if not isinstance(policy, dict):
                            return CheckResult.UNKNOWN
                        if policy.get('PolicyDocument'):
                            result = check_policy(policy['PolicyDocument'])
                            if result == CheckResult.FAILED:
                                return result
                    return CheckResult.PASSED
                # not empty and had non failing policies
                return CheckResult.UNKNOWN
        # this is just for Policy resources
        if isinstance(myproperties, dict) and 'PolicyDocument' in myproperties.keys():
            return check_policy(myproperties['PolicyDocument'])
        return CheckResult.UNKNOWN


check = IAMStarActionPolicyDocument()


def check_policy(policy_block):
    if policy_block:
        if isinstance(policy_block, str):
            policy_block = ast.literal_eval(policy_block)
        if isinstance(policy_block, dict) and 'Statement' in policy_block.keys():
            for statement in force_list(policy_block['Statement']):
                if 'Action' in statement and statement.get('Effect', ['Allow']) == 'Allow' and '*' in force_list(
                        statement['Action']):
                    return CheckResult.FAILED
                return CheckResult.PASSED
        else:
            return CheckResult.PASSED
    else:
        return CheckResult.PASSED



from __future__ import annotations

import logging
from typing import Any

from yaml import YAMLError

import checkov.common.parsers.yaml.loader as loader

logger = logging.getLogger(__name__)


def parse(
    filename: str, file_content: str | None = None
) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
    template = None
    template_lines = None
    try:
        if filename.endswith(".yaml") or filename.endswith(".yml"):
            template, template_lines = loader.load(filename, file_content)

        if template and template_lines:
            if isinstance(template, list):
                for t in template:
                    if t and isinstance(t, (list, dict)):
                        return t, template_lines
            else:
                return None
        else:
            return None
    except IOError as e:
        if e.errno == 2:
            logger.error(f"Template file not found: {filename}")
            return None
        elif e.errno == 21:
            logger.error(f"Template references a directory, not a file: {filename}")
            return None
        elif e.errno == 13:
            logger.error(f"Permission denied when accessing template file: {filename}")
            return None
    except UnicodeDecodeError:
        logger.error(f"Cannot read file contents: {filename}")
        return None
    except YAMLError:
        if filename.endswith(".yaml") or filename.endswith(".yml"):
            logger.debug(f"Cannot read file contents: {filename} - is it a yaml?")
        return None

    return None



from checkov.common.checks_infra.solvers.connections_solvers.and_connection_solver import AndConnectionSolver  # noqa
from checkov.common.checks_infra.solvers.connections_solvers.complex_connection_solver import ComplexConnectionSolver  # noqa
from checkov.common.checks_infra.solvers.connections_solvers.connection_exists_solver import ConnectionExistsSolver  # noqa
from checkov.common.checks_infra.solvers.connections_solvers.connection_not_exists_solver import ConnectionNotExistsSolver  # noqa
from checkov.common.checks_infra.solvers.connections_solvers.or_connection_solver import OrConnectionSolver  # noqa



from __future__ import annotations


class BreadcrumbMetadata:
    __slots__ = ("attribute_key", "vertex_id")

    def __init__(self, vertex_id: int, attribute_key: str | None):
        self.vertex_id = vertex_id
        self.attribute_key = attribute_key




from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class SecretManagerSecret90days(BaseResourceCheck):

    def __init__(self):
        name = "Ensure Secrets Manager secrets should be rotated within 90 days"
        id = "CKV_AWS_304"
        supported_resources = ["aws_secretsmanager_secret_rotation"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get("rotation_rules") and isinstance(conf.get("rotation_rules"), list):
            rule = conf.get("rotation_rules")[0]
            if rule.get('automatically_after_days') and isinstance(rule.get('automatically_after_days'), list):
                days = rule.get('automatically_after_days')[0]
                if days < 90:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = SecretManagerSecret90days()



from __future__ import annotations

import json
import logging
import os
import platform
from pathlib import Path
from typing import Any, Tuple
import dpath

import yaml
from jsonschema import validate, ValidationError

from checkov.common.parsers.yaml.loader import SafeLineLoaderGhaSchema
from checkov.common.parsers.yaml.parser import parse
from checkov.common.util.type_forcers import force_dict
from checkov.github_actions.graph_builder.graph_components.resource_types import ResourceType
from checkov.github_actions.schemas import gha_schema, gha_workflow
from checkov.runner_filter import RunnerFilter

WORKFLOW_DIRECTORY = ".github/workflows/"
WIN_WORKFLOW_DIRECTORY = ".github\\workflows\\"


def get_scannable_file_paths(root_folder: str | Path) -> set[Path]:
    """Finds yaml files"""

    file_paths: set[Path] = set()

    if root_folder:
        root_path = root_folder if isinstance(root_folder, Path) else Path(root_folder)
        file_paths = {file_path for file_path in root_path.rglob("*.[y][am]*[l]") if file_path.is_file()}

    return file_paths


def parse_file(
    f: str | Path, file_content: str | None = None
) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
    file_path = f if isinstance(f, Path) else Path(f)

    if is_workflow_file(file_path):
        if not file_content:
            file_content = file_path.read_text()

        entity_schema = parse(filename=str(f), file_content=file_content)

        if entity_schema and is_schema_valid(yaml.load(file_content, Loader=SafeLineLoaderGhaSchema)):  # nosec
            return entity_schema
    return None


def is_workflow_file(file_path: str | Path) -> bool:
    """
    :return: True if the file mentioned is in a github action workflow directory and is a YAML file. Otherwise: False
    """
    abspath = os.path.abspath(file_path)
    return get_workflow_dir() in abspath and abspath.endswith(("yml", "yaml"))


def get_workflow_dir() -> str:
    """
    Detects os and uses different dir string
    """
    if platform.system() == "Windows":
        return WIN_WORKFLOW_DIRECTORY
    return WORKFLOW_DIRECTORY


def is_schema_valid(config: dict[str, Any] | list[dict[str, Any]]) -> bool:
    config_dict = force_dict(config)

    try:
        validate(config_dict, gha_workflow)
        return True
    except ValidationError:
        try:
            validate(config_dict, gha_schema)
            return True
        except ValidationError:
            logging.info(
                "Given entity configuration does not match the schema\n" f"config={json.dumps(config_dict, indent=4)}\n"
            )

    return False


def get_gha_files_definitions(root_folder: str | Path,
                              files: "list[str] | None" = None,
                              runner_filter: RunnerFilter | None = None,) -> tuple[dict[str, Any], dict[str, Any]]:
    definitions = {}
    definitions_raw = {}
    file_paths = get_scannable_file_paths(root_folder=root_folder)
    files_set = set(files) if files else set()

    for file_path in file_paths:
        str_file_path = str(file_path)
        should_parse: bool = str_file_path in files_set if files_set else True
        if should_parse:
            result = parse_file(f=file_path)
            # result should be tuple of dict representing the file payload structure and list of lines of the payload
            if result is not None:
                definitions[str_file_path] = result[0]
                definitions_raw[str_file_path] = result[1]

    return definitions, definitions_raw


def build_gha_definitions_context(definitions: dict[str, dict[str, Any]], definitions_raw: dict[str, list[Tuple[int, str]]]) -> dict[str, dict[str, Any]]:
    definitions_context: dict[str, dict[str, Any]] = {}
    resources = [e.value for e in ResourceType]
    # iterate on the files
    for file_path, file_path_definitions in definitions.items():
        # iterate on the definitions (Parameters, Resources, Outputs...)
        for file_path_definition, definition in file_path_definitions.items():
            if isinstance(file_path_definition, str) and file_path_definition in resources:
                # iterate on the actual objects of each definition
                if isinstance(definition, dict):
                    for attribute, attr_value in definition.items():
                        if isinstance(attr_value, dict):
                            start_line = attr_value['__startline__']
                            end_line = attr_value['__endline__']
                        elif isinstance(attr_value, str) and '__startline__' in definition and '__endline__' in definition:
                            start_line = definition['__startline__']
                            end_line = definition['__endline__']
                        else:
                            continue

                        code_lines = definitions_raw[file_path][start_line - 1: end_line - 1]
                        dpath.new(
                            definitions_context,
                            [file_path, str(file_path_definition), str(attribute)],
                            {"start_line": start_line, "end_line": end_line, "code_lines": code_lines},
                        )
                elif isinstance(definition, (str, list)):
                    for line_tuple in definitions_raw[file_path]:
                        if file_path_definition in line_tuple[1] and definition_locator_helper(definition, line_tuple[1]):
                            code_lines = definitions_raw[file_path][line_tuple[0] - 1:line_tuple[0]]
                            dpath.new(
                                definitions_context,
                                [file_path, str(file_path_definition), str(definition)],
                                {"start_line": line_tuple[0], "end_line": line_tuple[0] + 1, "code_lines": code_lines},
                            )
                            break

    return definitions_context


def definition_locator_helper(definition: str | list[str], target: str) -> bool:
    if isinstance(definition, str):
        return definition in target
    elif isinstance(definition, list):
        return all(item in target for item in definition)
    return False



from __future__ import annotations

import logging
import typing
from copy import deepcopy
from typing import Any, Optional

from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.foreach.abstract_handler import ForeachAbstractHandler
from checkov.terraform.graph_builder.foreach.consts import FOREACH_STRING, COUNT_STRING, FOR_EACH_BLOCK_TYPE
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock

if typing.TYPE_CHECKING:
    from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph


class ForeachResourceHandler(ForeachAbstractHandler):
    def __init__(self, local_graph: TerraformLocalGraph) -> None:
        super().__init__(local_graph)

    def handle(self, resources_blocks: list[int]) -> None:
        block_index_to_statement: FOR_EACH_BLOCK_TYPE = self._get_statements(resources_blocks)
        self._create_new_resources(block_index_to_statement)

    def _get_statements(self, resources_blocks: list[int]) -> FOR_EACH_BLOCK_TYPE:
        if not resources_blocks:
            return {}
        block_index_to_statement: FOR_EACH_BLOCK_TYPE = {}
        for block_index, block in enumerate(self.local_graph.vertices):
            if block.block_type != BlockType.RESOURCE or not (FOREACH_STRING in block.attributes or COUNT_STRING in block.attributes):
                continue
            foreach_statement = self._get_static_foreach_statement(block_index)
            block_index_to_statement[block_index] = foreach_statement
        blocks_to_render = [block_idx for block_idx, statement in block_index_to_statement.items() if statement is None]
        if blocks_to_render:
            rendered_statements: FOR_EACH_BLOCK_TYPE = self._handle_dynamic_statement(blocks_to_render)
            block_index_to_statement.update(rendered_statements)
        return block_index_to_statement

    def _get_static_foreach_statement(self, block_index: int) -> Optional[list[str] | dict[str, Any] | int]:
        attributes = self.local_graph.vertices[block_index].attributes
        if not attributes.get(FOREACH_STRING) and not attributes.get(COUNT_STRING):
            return None
        try:
            if self._is_static_statement(block_index):
                return self._handle_static_statement(block_index)
            else:
                return None
        except Exception as e:
            logging.info(
                f"Cant get foreach statement for block: {self.local_graph.vertices[block_index]}, error: {str(e)}")
            return None

    def _handle_dynamic_statement(self, blocks_to_render: list[int]) -> FOR_EACH_BLOCK_TYPE:
        rendered_statements_by_idx: FOR_EACH_BLOCK_TYPE = {}
        sub_graph = self._build_sub_graph(blocks_to_render)
        self._render_sub_graph(sub_graph, blocks_to_render)
        for block_idx in blocks_to_render:
            if not self._is_static_statement(block_idx, sub_graph):
                rendered_statements_by_idx[block_idx] = None
            else:
                rendered_statements_by_idx[block_idx] = self._handle_static_statement(block_idx, sub_graph)
        return rendered_statements_by_idx

    def _create_new_resources_count(self, statement: int, block_idx: int) -> None:
        main_resource = self.local_graph.vertices[block_idx]
        for i in range(statement):
            self._create_new_resource(main_resource, i, resource_idx=block_idx, foreach_idx=i)

    def _create_new_resource(
            self,
            main_resource: TerraformBlock,
            new_value: int | str,
            resource_idx: int,
            foreach_idx: int,
            new_key: int | str | None = None,
    ) -> None:
        new_resource = deepcopy(main_resource)
        block_type, block_name = new_resource.name.split('.')
        key_to_val_changes = self._build_key_to_val_changes(main_resource, new_value, new_key)
        config_attrs = new_resource.config.get(block_type, {}).get(block_name, {})

        self._update_foreach_attrs(config_attrs, key_to_val_changes, new_resource)
        idx_to_change = new_key or new_value
        self._add_index_to_resource_block_properties(new_resource, idx_to_change)
        if foreach_idx == 0:
            self.local_graph.vertices[resource_idx] = new_resource
        else:
            self.local_graph.vertices.append(new_resource)

    def _create_new_foreach_resource(self, block_idx: int, foreach_idx: int, main_resource: TerraformBlock,
                                     new_key: int | str, new_value: int | str) -> None:
        self._create_new_resource(main_resource, new_value, new_key=new_key, resource_idx=block_idx,
                                  foreach_idx=foreach_idx)

    @staticmethod
    def _add_index_to_resource_block_properties(block: TerraformBlock, idx: str | int) -> None:
        block_type, block_name = block.name.split('.')
        idx_with_separator = ForeachResourceHandler._update_block_name_and_id(block, idx)
        if block.config.get(block_type) and block.config.get(block_type, {}).get(block_name):
            block.config[block_type][f"{block_name}[{idx_with_separator}]"] = block.config[block_type].pop(block_name)

    def _create_new_resources(self, block_index_to_statement: FOR_EACH_BLOCK_TYPE) -> None:
        for block_idx, statement in block_index_to_statement.items():
            if not statement:
                continue
            if isinstance(statement, int):
                self._create_new_resources_count(statement, block_idx)
            else:
                self._create_new_resources_foreach(statement, block_idx)



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class MongoDBIsPublic(BaseResourceCheck):
    def __init__(self):
        name = "Ensure MongoDB instance is not public"
        id = "CKV_ALI_43"
        supported_resources = ['alicloud_mongodb_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        security_ip_list = conf.get("security_ip_list")
        if security_ip_list and isinstance(security_ip_list, list):
            addresses = security_ip_list[0]
            if "0.0.0.0" in addresses or "0.0.0.0/0" in addresses:  # nosec B104
                self.evaluated_keys = ['security_ip_list']
                return CheckResult.FAILED
        return CheckResult.PASSED


check = MongoDBIsPublic()



from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any


# NOTE: These would be better as TypedDict, but that requires python 3.8 :-(


@dataclass
class VarReference:
    definition_name: str               # Example: 'region'
    definition_expression: str         # Example: '${var.region}'
    definition_path: str               # Example: 'resource/0/aws_s3_bucket/foo-bucket/region/0'

    # Make the object subscriptable for backwards compatibility to when a simple dict was used
    def __getitem__(self, item: str) -> str:
        if not isinstance(item, str):
            raise TypeError("Item key must be a str")

        if item == "definition_name":
            return self.definition_name
        if item == "definition_expression":
            return self.definition_expression
        if item == "definition_path":
            return self.definition_path

        raise KeyError(f"Unknown key: {item}")


@dataclass
class EvaluationContext:
    var_file: str                     # File the variable was defined in (e.g., '/tf/example.tf'
    value: Any = None                 # Example: 'us-east-1'
    definitions: list[VarReference] = field(default_factory=list)

    # Make the object subscriptable for backwards compatibility to when a simple dict was used
    def __getitem__(self, item: str) -> Any:
        if not isinstance(item, str):
            raise TypeError("Item key must be a str")

        if item == "var_file":
            return self.var_file
        if item == "value":
            return self.value
        if item == "definitions":
            return self.definitions

        raise KeyError(f"Unknown key: {item}")



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from typing import Any


class AllowPrivilegeEscalationPSP(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        # CIS-1.3 1.7.5
        # CIS-1.5 5.2.5
        # https://kubernetes.io/docs/tasks/configure-pod-container/security-context/
        # AllowPrivilegeEscalation is true always when the container is: 1) run as Privileged OR 2) has CAP_SYS_ADMIN.
        # https://kubernetes.io/docs/concepts/policy/pod-security-policy/
        # AllowPrivilegeEscalation - This defaults to allow to not break setuid binaries
        # DefaultAllowPrivilegeEscalation - Default is to allow as to not breat setuid binaries
        # If you omit allowPrivilegeEscalation from PSP, it defaults to true
        # Location: PodSecurityPolicy.spec.allowPrivilegeEscalation
        name = "Containers should not run with allowPrivilegeEscalation"
        id = "CKV_K8S_5"
        supported_resources = ('kubernetes_pod_security_policy',)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'spec/[0]/allow_privilege_escalation'

    def get_forbidden_values(self) -> list[Any]:
        return [True]


check = AllowPrivilegeEscalationPSP()



from __future__ import annotations

from ipaddress import ip_network, ip_address
from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class VnetLocalDNS(BaseResourceCheck):
    def __init__(self) -> None:
        """Avoid taking a dependency on external DNS servers
         for local communication such as those deployed on-premises.
        Where possible consider deploying Azure Private DNS Zones,
         a platform-as-a-service (PaaS) DNS service for VNETs"""
        name = "Ensure that VNET uses local DNS addresses"
        id = "CKV_AZURE_183"
        supported_resources = ("azurerm_virtual_network",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if "dns_servers" in conf and conf["dns_servers"] and isinstance(conf["dns_servers"], list):
            dns_servers = conf["dns_servers"][0]
            if dns_servers:
                for ip in dns_servers:
                    if "address_space" in conf and conf["address_space"] and isinstance(conf["address_space"], list):
                        address_spaces = conf["address_space"][0]
                        if isinstance(address_spaces, list):
                            for address_range in address_spaces:
                                if not isinstance(address_range, str):
                                    continue
                                try:
                                    net = ip_network(address_range)
                                    ip_add = ip_address(ip) if isinstance(ip, str) else None
                                except ValueError:
                                    return CheckResult.UNKNOWN
                                if isinstance(ip, str) and ip_add in net:
                                    return CheckResult.PASSED
                self.evaluated_keys = ["dns_servers"]
                return CheckResult.FAILED
        return CheckResult.PASSED


check = VnetLocalDNS()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class LaunchConfigurationEBSEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all data stored in the Launch configuration EBS is securely encrypted"
        id = "CKV_AWS_8"
        supported_resources = ['AWS::AutoScaling::LaunchConfiguration']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
        Looks for encryption configuration of device block mapping in an AWS launch configurations
        https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-template.html
        :param conf: aws_launch_configuration configuration
        :return: <CheckResult>
        """
        properties = conf.get('Properties', {})
        if properties is None:
            return CheckResult.UNKNOWN
        block_device_mappings = properties.get('BlockDeviceMappings')
        if block_device_mappings is None:
            return CheckResult.UNKNOWN
        if not isinstance(block_device_mappings, list):
            return CheckResult.UNKNOWN
        for block_device_mapping in block_device_mappings:
            if not isinstance(block_device_mapping, dict):
                return CheckResult.UNKNOWN
            if block_device_mapping.get('Ebs'):
                if not block_device_mapping['Ebs'].get('Encrypted'):
                    return CheckResult.FAILED

        return CheckResult.PASSED


check = LaunchConfigurationEBSEncryption()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class WAFRuleHasAnyActions(BaseResourceCheck):
    def __init__(self):
        name = "Ensure WAF rule has any actions"
        id = "CKV_AWS_342"
        supported_resources = ('aws_waf_web_acl', 'aws_wafregional_web_acl', 'aws_wafv2_web_acl',
                               'aws_wafv2_rule_group', 'aws_wafregional_rule_group', 'aws_waf_rule_group')
        categories = (CheckCategories.APPLICATION_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        rules = None
        if conf.get("rule") and isinstance(conf["rule"], list):
            rules = conf["rule"]
        # dont blame me i didn't name one rule and the other rules and the other activated_rule
        elif conf.get("rules") and isinstance(conf["rules"], list):
            rules = conf["rules"]
        elif conf.get("activated_rule") and isinstance(conf["activated_rule"], list):
            rules = conf["activated_rule"]

        if isinstance(rules, list):
            for rule in rules:
                if "action" in rule and rule['action'] != [{}]:
                    continue
                if "override_action" in rule and rule['override_action'] != [{}]:
                    continue
                return CheckResult.FAILED
            return CheckResult.PASSED

        return CheckResult.UNKNOWN


check = WAFRuleHasAnyActions()
